@@ -1,4 +1,7 @@
+from typing import TYPE_CHECKING, List
+
 from dagster import check
+from dagster.core.errors import DagsterUserCodeProcessError
 from dagster.core.host_representation.external_data import (
     ExternalPartitionConfigData,
     ExternalPartitionExecutionErrorData,
@@ -8,75 +11,50 @@ from dagster.core.host_representation.external_data import (
 )
 from dagster.core.host_representation.handle import RepositoryHandle
 from dagster.grpc.types import PartitionArgs, PartitionNamesArgs, PartitionSetExecutionParamArgs
+from dagster.serdes import deserialize_as
 
-from .utils import execute_unary_api_cli_command
+if TYPE_CHECKING:
+    from dagster.grpc.client import DagsterGrpcClient
 
 
-def sync_get_external_partition_names(repository_handle, partition_set_name):
-    check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
-    check.str_param(partition_set_name, "partition_set_name")
-    repository_origin = repository_handle.get_origin()
-
-    return check.inst(
-        execute_unary_api_cli_command(
-            repository_origin.executable_path,
-            "partition_names",
-            PartitionNamesArgs(
-                repository_origin=repository_origin, partition_set_name=partition_set_name
-            ),
-        ),
-        (ExternalPartitionNamesData, ExternalPartitionExecutionErrorData),
-    )
-
-
-def sync_get_external_partition_names_grpc(api_client, repository_handle, partition_set_name):
+def sync_get_external_partition_names_grpc(
+    api_client: "DagsterGrpcClient", repository_handle: RepositoryHandle, partition_set_name: str
+) -> ExternalPartitionNamesData:
     from dagster.grpc.client import DagsterGrpcClient
 
     check.inst_param(api_client, "api_client", DagsterGrpcClient)
     check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
     check.str_param(partition_set_name, "partition_set_name")
-    repository_origin = repository_handle.get_origin()
-    return check.inst(
+    repository_origin = repository_handle.get_external_origin()
+    result = deserialize_as(
         api_client.external_partition_names(
             partition_names_args=PartitionNamesArgs(
-                repository_origin=repository_origin, partition_set_name=partition_set_name,
+                repository_origin=repository_origin,
+                partition_set_name=partition_set_name,
             ),
         ),
         (ExternalPartitionNamesData, ExternalPartitionExecutionErrorData),
     )
+    if isinstance(result, ExternalPartitionExecutionErrorData):
+        raise DagsterUserCodeProcessError.from_error_info(result.error)
 
-
-def sync_get_external_partition_config(repository_handle, partition_set_name, partition_name):
-    check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
-    check.str_param(partition_set_name, "partition_set_name")
-    check.str_param(partition_name, "partition_name")
-    repository_origin = repository_handle.get_origin()
-
-    return check.inst(
-        execute_unary_api_cli_command(
-            repository_origin.executable_path,
-            "partition_config",
-            PartitionArgs(
-                repository_origin=repository_origin,
-                partition_set_name=partition_set_name,
-                partition_name=partition_name,
-            ),
-        ),
-        (ExternalPartitionConfigData, ExternalPartitionExecutionErrorData),
-    )
+    return result
 
 
 def sync_get_external_partition_config_grpc(
-    api_client, repository_handle, partition_set_name, partition_name
-):
+    api_client: "DagsterGrpcClient",
+    repository_handle: RepositoryHandle,
+    partition_set_name: str,
+    partition_name: str,
+) -> ExternalPartitionConfigData:
     from dagster.grpc.client import DagsterGrpcClient
 
     check.inst_param(api_client, "api_client", DagsterGrpcClient)
     check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
     check.str_param(partition_set_name, "partition_set_name")
     check.str_param(partition_name, "partition_name")
-    repository_origin = repository_handle.get_origin()
-    return check.inst(
+    repository_origin = repository_handle.get_external_origin()
+    result = deserialize_as(
         api_client.external_partition_config(
             partition_args=PartitionArgs(
                 repository_origin=repository_origin,
@@ -86,31 +64,18 @@ def sync_get_external_partition_config_grpc(
         ),
         (ExternalPartitionConfigData, ExternalPartitionExecutionErrorData),
     )
+    if isinstance(result, ExternalPartitionExecutionErrorData):
+        raise DagsterUserCodeProcessError.from_error_info(result.error)
 
-
-def sync_get_external_partition_tags(repository_handle, partition_set_name, partition_name):
-    check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
-    check.str_param(partition_set_name, "partition_set_name")
-    check.str_param(partition_name, "partition_name")
-    repository_origin = repository_handle.get_origin()
-
-    return check.inst(
-        execute_unary_api_cli_command(
-            repository_origin.executable_path,
-            "partition_tags",
-            PartitionArgs(
-                repository_origin=repository_origin,
-                partition_set_name=partition_set_name,
-                partition_name=partition_name,
-            ),
-        ),
-        (ExternalPartitionTagsData, ExternalPartitionExecutionErrorData),
-    )
+    return result
 
 
 def sync_get_external_partition_tags_grpc(
-    api_client, repository_handle, partition_set_name, partition_name
-):
+    api_client: "DagsterGrpcClient",
+    repository_handle: RepositoryHandle,
+    partition_set_name: str,
+    partition_name: str,
+) -> ExternalPartitionTagsData:
     from dagster.grpc.client import DagsterGrpcClient
 
     check.inst_param(api_client, "api_client", DagsterGrpcClient)
@@ -118,8 +83,8 @@ def sync_get_external_partition_tags_grpc(
     check.str_param(partition_set_name, "partition_set_name")
     check.str_param(partition_name, "partition_name")
 
-    repository_origin = repository_handle.get_origin()
-    return check.inst(
+    repository_origin = repository_handle.get_external_origin()
+    result = deserialize_as(
         api_client.external_partition_tags(
             partition_args=PartitionArgs(
                 repository_origin=repository_origin,
@@ -129,34 +94,18 @@ def sync_get_external_partition_tags_grpc(
         ),
         (ExternalPartitionTagsData, ExternalPartitionExecutionErrorData),
     )
+    if isinstance(result, ExternalPartitionExecutionErrorData):
+        raise DagsterUserCodeProcessError.from_error_info(result.error)
 
-
-def sync_get_external_partition_set_execution_param_data(
-    repository_handle, partition_set_name, partition_names
-):
-    check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
-    check.str_param(partition_set_name, "partition_set_name")
-    check.list_param(partition_names, "partition_names", of_type=str)
-
-    repository_origin = repository_handle.get_origin()
-
-    return check.inst(
-        execute_unary_api_cli_command(
-            repository_origin.executable_path,
-            "partition_set_execution_param_data",
-            PartitionSetExecutionParamArgs(
-                repository_origin=repository_origin,
-                partition_set_name=partition_set_name,
-                partition_names=partition_names,
-            ),
-        ),
-        (ExternalPartitionSetExecutionParamData, ExternalPartitionExecutionErrorData),
-    )
+    return result
 
 
 def sync_get_external_partition_set_execution_param_data_grpc(
-    api_client, repository_handle, partition_set_name, partition_names
-):
+    api_client: "DagsterGrpcClient",
+    repository_handle: RepositoryHandle,
+    partition_set_name: str,
+    partition_names: List[str],
+) -> ExternalPartitionSetExecutionParamData:
     from dagster.grpc.client import DagsterGrpcClient
 
     check.inst_param(api_client, "api_client", DagsterGrpcClient)
@@ -164,9 +113,9 @@ def sync_get_external_partition_set_execution_param_data_grpc(
     check.str_param(partition_set_name, "partition_set_name")
     check.list_param(partition_names, "partition_names", of_type=str)
 
-    repository_origin = repository_handle.get_origin()
+    repository_origin = repository_handle.get_external_origin()
 
-    return check.inst(
+    result = deserialize_as(
         api_client.external_partition_set_execution_params(
             partition_set_execution_param_args=PartitionSetExecutionParamArgs(
                 repository_origin=repository_origin,
@@ -176,3 +125,7 @@ def sync_get_external_partition_set_execution_param_data_grpc(
         ),
         (ExternalPartitionSetExecutionParamData, ExternalPartitionExecutionErrorData),
     )
+    if isinstance(result, ExternalPartitionExecutionErrorData):
+        raise DagsterUserCodeProcessError.from_error_info(result.error)
+
+    return result
