@@ -2,47 +2,45 @@ import os
 import re
 import sys
 import textwrap
+from typing import Any, Callable, Dict, List, Mapping, Optional, Tuple, cast
 
 import click
 import pendulum
 import yaml
-from dagster import PipelineDefinition, check, execute_pipeline
+from tabulate import tabulate
+
+from dagster import PipelineDefinition
+from dagster import __version__ as dagster_version
+from dagster import check, execute_pipeline
 from dagster.cli.workspace.cli_target import (
     WORKSPACE_TARGET_WARNING,
-    get_external_pipeline_from_external_repo,
-    get_external_pipeline_from_kwargs,
+    get_external_pipeline_or_job_from_external_repo,
+    get_external_pipeline_or_job_from_kwargs,
     get_external_repository_from_kwargs,
     get_external_repository_from_repo_location,
-    get_pipeline_python_origin_from_kwargs,
-    get_repository_location_from_kwargs,
+    get_pipeline_or_job_python_origin_from_kwargs,
+    get_repository_location_from_workspace,
+    get_workspace_from_kwargs,
     pipeline_target_argument,
-    python_pipeline_config_argument,
+    python_pipeline_or_job_config_argument,
     python_pipeline_target_argument,
     repository_target_argument,
 )
 from dagster.core.definitions.pipeline_base import IPipeline
-from dagster.core.errors import (
-    DagsterBackfillFailedError,
-    DagsterInvariantViolationError,
-    DagsterLaunchFailedError,
-)
+from dagster.core.errors import DagsterBackfillFailedError, DagsterInvariantViolationError
 from dagster.core.execution.api import create_execution_plan
 from dagster.core.execution.backfill import BulkActionStatus, PartitionBackfill, create_backfill_run
-from dagster.core.execution.resolve_versions import resolve_memoized_execution_plan
 from dagster.core.host_representation import (
     ExternalPipeline,
     ExternalRepository,
     RepositoryHandle,
     RepositoryLocation,
 )
-from dagster.core.host_representation.external_data import (
-    ExternalPartitionExecutionErrorData,
-    ExternalPartitionSetExecutionParamData,
-)
+from dagster.core.host_representation.external_data import ExternalPartitionSetExecutionParamData
 from dagster.core.host_representation.selector import PipelineSelector
 from dagster.core.instance import DagsterInstance
 from dagster.core.snap import PipelineSnapshot, SolidInvocationSnap
-from dagster.core.snap.execution_plan_snapshot import ExecutionPlanSnapshotErrorData
+from dagster.core.storage.tags import MEMOIZED_RUN_TAG
 from dagster.core.telemetry import log_external_repo_stats, telemetry_wrapper
 from dagster.core.utils import make_new_backfill_id
 from dagster.seven import IS_WINDOWS, JSONDecodeError, json
@@ -51,21 +49,16 @@ from dagster.utils.error import serializable_error_info_from_exc_info
 from dagster.utils.hosted_user_process import recon_pipeline_from_origin
 from dagster.utils.indenting_printer import IndentingPrinter
 from dagster.utils.interrupts import capture_interrupts
-from tabulate import tabulate
 
 from .config_scaffolder import scaffold_pipeline_config
+from .utils import get_instance_for_service
 
 
-def create_pipeline_cli_group():
-    group = click.Group(name="pipeline")
-    group.add_command(pipeline_list_command)
-    group.add_command(pipeline_print_command)
-    group.add_command(pipeline_execute_command)
-    group.add_command(pipeline_backfill_command)
-    group.add_command(pipeline_scaffold_command)
-    group.add_command(pipeline_launch_command)
-    group.add_command(pipeline_list_versions_command)
-    return group
+@click.group(name="pipeline")
+def pipeline_cli():
+    """
+    Commands for working with Dagster pipelines/jobs.
+    """
 
 
 def apply_click_params(command, *click_params):
@@ -74,40 +67,56 @@ def apply_click_params(command, *click_params):
     return command
 
 
-@click.command(
+@pipeline_cli.command(
     name="list",
-    help="List the pipelines in a repository. {warning}".format(warning=WORKSPACE_TARGET_WARNING),
+    help="List the pipelines/jobs in a repository. {warning}".format(
+        warning=WORKSPACE_TARGET_WARNING
+    ),
 )
 @repository_target_argument
 def pipeline_list_command(**kwargs):
-    with DagsterInstance.get() as instance:
-        return execute_list_command(kwargs, click.echo, instance)
+    return execute_list_command(kwargs, click.echo)
 
 
-def execute_list_command(cli_args, print_fn, instance):
-    check.inst_param(instance, "instance", DagsterInstance)
-    with get_external_repository_from_kwargs(cli_args) as external_repository:
-        title = "Repository {name}".format(name=external_repository.name)
-        print_fn(title)
-        print_fn("*" * len(title))
-        first = True
-        for pipeline in external_repository.get_all_external_pipelines():
-            pipeline_title = "Pipeline: {name}".format(name=pipeline.name)
+def execute_list_command(cli_args, print_fn, using_job_op_graph_apis=False):
+    with get_instance_for_service(
+        "``dagster job list``" if using_job_op_graph_apis else "``dagster pipeline list``"
+    ) as instance:
+        with get_external_repository_from_kwargs(
+            instance, version=dagster_version, kwargs=cli_args
+        ) as external_repository:
+            title = "Repository {name}".format(name=external_repository.name)
+            print_fn(title)
+            print_fn("*" * len(title))
+            first = True
+            for pipeline in (
+                external_repository.get_external_jobs()
+                if using_job_op_graph_apis
+                else external_repository.get_all_external_pipelines()
+            ):
+                pipeline_title = "{pipeline_or_job}: {name}".format(
+                    pipeline_or_job="Job" if using_job_op_graph_apis else "Pipeline",
+                    name=pipeline.name,
+                )
 
-            if not first:
-                print_fn("*" * len(pipeline_title))
-            first = False
+                if not first:
+                    print_fn("*" * len(pipeline_title))
+                first = False
 
-            print_fn(pipeline_title)
-            if pipeline.description:
-                print_fn("Description:")
-                print_fn(format_description(pipeline.description, indent=" " * 4))
-            print_fn("Solids: (Execution Order)")
-            for solid_name in pipeline.pipeline_snapshot.solid_names_in_topological_order:
-                print_fn("    " + solid_name)
+                print_fn(pipeline_title)
+                if pipeline.description:
+                    print_fn("Description:")
+                    print_fn(format_description(pipeline.description, indent=" " * 4))
+                print_fn(
+                    "{solid_or_op}: (Execution Order)".format(
+                        solid_or_op="Ops" if using_job_op_graph_apis else "Solids"
+                    )
+                )
+                for solid_name in pipeline.pipeline_snapshot.solid_names_in_topological_order:
+                    print_fn("    " + solid_name)
 
 
-def format_description(desc, indent):
+def format_description(desc: str, indent: str):
     check.str_param(desc, "desc")
     check.str_param(indent, "indent")
     desc = re.sub(r"\s+", " ", desc)
@@ -119,7 +128,7 @@ def format_description(desc, indent):
 
 def get_pipeline_in_same_python_env_instructions(command_name):
     return (
-        "This commands targets a pipeline. The pipeline can be specified in a number of ways:"
+        "This commands targets a pipeline/job. The pipeline/job can be specified in a number of ways:"
         "\n\n1. dagster pipeline {command_name} -f /path/to/file.py -a define_some_pipeline"
         "\n\n2. dagster pipeline {command_name} -m a_module.submodule -a define_some_pipeline"
         "\n\n3. dagster pipeline {command_name} -f /path/to/file.py -a define_some_repo -p <<pipeline_name>>"
@@ -141,7 +150,7 @@ def get_pipeline_instructions(command_name):
 
 def get_partitioned_pipeline_instructions(command_name):
     return (
-        "This commands targets a partitioned pipeline. The pipeline and partition set must be "
+        "This commands targets a partitioned pipeline/job. The pipeline/job and partition set must be "
         "defined in a repository, which can be specified in a number of ways:"
         "\n\n1. dagster pipeline {command_name} -p <<pipeline_name>> (works if .{default_filename} exists)"
         "\n\n2. dagster pipeline {command_name} -p <<pipeline_name>> -w path/to/{default_filename}"
@@ -150,53 +159,74 @@ def get_partitioned_pipeline_instructions(command_name):
     ).format(command_name=command_name, default_filename=DEFAULT_WORKSPACE_YAML_FILENAME)
 
 
-@click.command(
+@pipeline_cli.command(
     name="print",
-    help="Print a pipeline.\n\n{instructions}".format(
+    help="Print a pipeline/job.\n\n{instructions}".format(
         instructions=get_pipeline_instructions("print")
     ),
 )
 @click.option("--verbose", is_flag=True)
 @pipeline_target_argument
 def pipeline_print_command(verbose, **cli_args):
-    with DagsterInstance.get():
-        return execute_print_command(verbose, cli_args, click.echo)
+    with DagsterInstance.get() as instance:
+        return execute_print_command(instance, verbose, cli_args, click.echo)
 
 
-def execute_print_command(verbose, cli_args, print_fn):
-    with get_external_pipeline_from_kwargs(cli_args) as external_pipeline:
+def execute_print_command(instance, verbose, cli_args, print_fn, using_job_op_graph_apis=False):
+    with get_external_pipeline_or_job_from_kwargs(
+        instance,
+        version=dagster_version,
+        kwargs=cli_args,
+        using_job_op_graph_apis=using_job_op_graph_apis,
+    ) as external_pipeline:
         pipeline_snapshot = external_pipeline.pipeline_snapshot
 
         if verbose:
-            print_pipeline(pipeline_snapshot, print_fn=print_fn)
+            print_pipeline_or_job(
+                pipeline_snapshot,
+                print_fn=print_fn,
+                using_job_op_graph_apis=using_job_op_graph_apis,
+            )
         else:
-            print_solids(pipeline_snapshot, print_fn=print_fn)
+            print_solids_or_ops(
+                pipeline_snapshot,
+                print_fn=print_fn,
+                using_job_op_graph_apis=using_job_op_graph_apis,
+            )
 
 
-def print_solids(pipeline_snapshot, print_fn):
+def print_solids_or_ops(
+    pipeline_snapshot: PipelineSnapshot,
+    print_fn: Callable[..., Any],
+    using_job_op_graph_apis: bool = False,
+):
     check.inst_param(pipeline_snapshot, "pipeline", PipelineSnapshot)
     check.callable_param(print_fn, "print_fn")
 
     printer = IndentingPrinter(indent_level=2, printer=print_fn)
-    printer.line("Pipeline: {name}".format(name=pipeline_snapshot.name))
+    printer.line(f"{'Job' if using_job_op_graph_apis else 'Pipeline'}: {pipeline_snapshot.name}")
 
-    printer.line("Solids:")
+    printer.line(f"{'Ops' if using_job_op_graph_apis else 'Solids'}")
     for solid in pipeline_snapshot.dep_structure_snapshot.solid_invocation_snaps:
         with printer.with_indent():
-            printer.line("Solid: {name}".format(name=solid.solid_name))
+            printer.line(f"{'Op' if using_job_op_graph_apis else 'Solid'}: {solid.solid_name}")
 
 
-def print_pipeline(pipeline_snapshot, print_fn):
+def print_pipeline_or_job(
+    pipeline_snapshot: PipelineSnapshot,
+    print_fn: Callable[..., Any],
+    using_job_op_graph_apis: bool = False,
+):
     check.inst_param(pipeline_snapshot, "pipeline", PipelineSnapshot)
     check.callable_param(print_fn, "print_fn")
     printer = IndentingPrinter(indent_level=2, printer=print_fn)
-    printer.line("Pipeline: {name}".format(name=pipeline_snapshot.name))
+    printer.line(f"{'Job' if using_job_op_graph_apis else 'Pipeline'}: {pipeline_snapshot.name}")
     print_description(printer, pipeline_snapshot.description)
 
-    printer.line("Solids:")
+    printer.line(f"{'Ops' if using_job_op_graph_apis else 'Solids'}")
     for solid in pipeline_snapshot.dep_structure_snapshot.solid_invocation_snaps:
         with printer.with_indent():
-            print_solid(printer, pipeline_snapshot, solid)
+            print_solid_or_op(printer, pipeline_snapshot, solid, using_job_op_graph_apis)
 
 
 def print_description(printer, desc):
@@ -207,10 +237,17 @@ def print_description(printer, desc):
                 printer.line(format_description(desc, printer.current_indent_str))
 
 
-def print_solid(printer, pipeline_snapshot, solid_invocation_snap):
+def print_solid_or_op(
+    printer: IndentingPrinter,
+    pipeline_snapshot: PipelineSnapshot,
+    solid_invocation_snap: SolidInvocationSnap,
+    using_job_op_graph_apis: bool,
+) -> None:
     check.inst_param(pipeline_snapshot, "pipeline_snapshot", PipelineSnapshot)
     check.inst_param(solid_invocation_snap, "solid_invocation_snap", SolidInvocationSnap)
-    printer.line("Solid: {name}".format(name=solid_invocation_snap.solid_name))
+    printer.line(
+        f"{'Op' if using_job_op_graph_apis else 'Solid'}: {solid_invocation_snap.solid_name}"
+    )
     with printer.with_indent():
         printer.line("Inputs:")
         for input_dep_snap in solid_invocation_snap.input_dep_snaps:
@@ -218,20 +255,20 @@ def print_solid(printer, pipeline_snapshot, solid_invocation_snap):
                 printer.line("Input: {name}".format(name=input_dep_snap.input_name))
 
         printer.line("Outputs:")
-        for output_def_snap in pipeline_snapshot.get_solid_def_snap(
+        for output_def_snap in pipeline_snapshot.get_node_def_snap(
             solid_invocation_snap.solid_def_name
         ).output_def_snaps:
             printer.line(output_def_snap.name)
 
 
-@click.command(
+@pipeline_cli.command(
     name="list_versions",
     help="Display the freshness of memoized results for the given pipeline.\n\n{instructions}".format(
         instructions=get_pipeline_in_same_python_env_instructions("list_versions")
     ),
 )
 @python_pipeline_target_argument
-@python_pipeline_config_argument("list_versions")
+@python_pipeline_or_job_config_argument("list_versions")
 @click.option(
     "--preset",
     type=click.STRING,
@@ -246,28 +283,37 @@ def pipeline_list_versions_command(**kwargs):
         execute_list_versions_command(instance, kwargs)
 
 
-def execute_list_versions_command(instance, kwargs):
+def execute_list_versions_command(instance: DagsterInstance, kwargs: Dict[str, Any]) -> None:
     check.inst_param(instance, "instance", DagsterInstance)
 
-    config = list(check.opt_tuple_param(kwargs.get("config"), "config", default=(), of_type=str))
+    config = list(
+        check.opt_tuple_param(kwargs.get("config"), "config", default=tuple(), of_type=str)
+    )
     preset = kwargs.get("preset")
     mode = kwargs.get("mode")
 
     if preset and config:
         raise click.UsageError("Can not use --preset with --config.")
 
-    pipeline_origin = get_pipeline_python_origin_from_kwargs(kwargs)
+    pipeline_origin = get_pipeline_or_job_python_origin_from_kwargs(kwargs)
     pipeline = recon_pipeline_from_origin(pipeline_origin)
     run_config = get_run_config_from_file_list(config)
-    execution_plan = create_execution_plan(
-        pipeline.get_definition(), run_config=run_config, mode=mode
+
+    memoized_plan = create_execution_plan(
+        pipeline,
+        run_config=run_config,
+        mode=mode,
+        instance_ref=instance.get_ref(),
+        tags={MEMOIZED_RUN_TAG: "true"},
     )
-    step_output_versions = execution_plan.resolve_step_output_versions()
-    memoized_plan = resolve_memoized_execution_plan(execution_plan, run_config, instance)
+    add_step_to_table(memoized_plan)
+
+
+def add_step_to_table(memoized_plan):
     # the step keys that we need to execute are those which do not have their inputs populated.
     step_keys_not_stored = set(memoized_plan.step_keys_to_execute)
     table = []
-    for step_output_handle, version in step_output_versions.items():
+    for step_output_handle, version in memoized_plan.step_output_versions.items():
         table.append(
             [
                 "{key}.{output}".format(
@@ -285,14 +331,14 @@ def execute_list_versions_command(instance, kwargs):
     click.echo(table_str)
 
 
-@click.command(
+@pipeline_cli.command(
     name="execute",
     help="Execute a pipeline.\n\n{instructions}".format(
         instructions=get_pipeline_in_same_python_env_instructions("execute")
     ),
 )
 @python_pipeline_target_argument
-@python_pipeline_config_argument("execute")
+@python_pipeline_or_job_config_argument("execute")
 @click.option(
     "--preset",
     type=click.STRING,
@@ -302,7 +348,9 @@ def execute_list_versions_command(instance, kwargs):
 @click.option(
     "--mode", type=click.STRING, help="The name of the mode in which to execute the pipeline."
 )
-@click.option("--tags", type=click.STRING, help="JSON string of tags to use for this pipeline run")
+@click.option(
+    "--tags", type=click.STRING, help="JSON string of tags to use for this pipeline/job run"
+)
 @click.option(
     "-s",
     "--solid-selection",
@@ -320,24 +368,26 @@ def execute_list_versions_command(instance, kwargs):
 )
 def pipeline_execute_command(**kwargs):
     with capture_interrupts():
-        with DagsterInstance.get() as instance:
+        with get_instance_for_service("``dagster pipeline execute``") as instance:
             execute_execute_command(instance, kwargs)
 
 
 @telemetry_wrapper
-def execute_execute_command(instance, kwargs):
+def execute_execute_command(
+    instance: DagsterInstance, kwargs: Dict[str, object], using_job_op_graph_apis: bool = False
+):
     check.inst_param(instance, "instance", DagsterInstance)
 
     config = list(check.opt_tuple_param(kwargs.get("config"), "config", default=(), of_type=str))
-    preset = kwargs.get("preset")
-    mode = kwargs.get("mode")
+    preset = cast(Optional[str], kwargs.get("preset"))
+    mode = cast(Optional[str], kwargs.get("mode"))
 
     if preset and config:
         raise click.UsageError("Can not use --preset with --config.")
 
     tags = get_tags_from_args(kwargs)
 
-    pipeline_origin = get_pipeline_python_origin_from_kwargs(kwargs)
+    pipeline_origin = get_pipeline_or_job_python_origin_from_kwargs(kwargs, using_job_op_graph_apis)
     pipeline = recon_pipeline_from_origin(pipeline_origin)
     solid_selection = get_solid_selection_from_args(kwargs)
     result = do_execute_command(pipeline, instance, config, mode, tags, solid_selection, preset)
@@ -348,14 +398,19 @@ def execute_execute_command(instance, kwargs):
     return result
 
 
-def get_run_config_from_file_list(file_list):
+def get_run_config_from_file_list(file_list: Optional[List[str]]):
     check.opt_list_param(file_list, "file_list", of_type=str)
     return load_yaml_from_glob_list(file_list) if file_list else {}
 
 
 def _check_execute_external_pipeline_args(
-    external_pipeline, run_config, mode, preset, tags, solid_selection
-):
+    external_pipeline: ExternalPipeline,
+    run_config: Dict[str, object],
+    mode: Optional[str],
+    preset: Optional[str],
+    tags: Optional[Mapping[str, object]],
+    solid_selection: Optional[List[str]],
+) -> Tuple[Dict[str, object], str, Mapping[str, object], Optional[List[str]]]:
     check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
     run_config = check.opt_dict_param(run_config, "run_config")
     check.opt_str_param(mode, "mode")
@@ -439,22 +494,22 @@ def _check_execute_external_pipeline_args(
 
 
 def _create_external_pipeline_run(
-    instance,
-    repo_location,
-    external_repo,
-    external_pipeline,
-    run_config,
-    mode,
-    preset,
-    tags,
-    solid_selection,
-    run_id,
+    instance: DagsterInstance,
+    repo_location: RepositoryLocation,
+    external_repo: ExternalRepository,
+    external_pipeline: ExternalPipeline,
+    run_config: Dict[str, object],
+    mode: Optional[str],
+    preset: Optional[str],
+    tags: Optional[Mapping[str, object]],
+    solid_selection: Optional[List[str]],
+    run_id: Optional[str],
 ):
     check.inst_param(instance, "instance", DagsterInstance)
     check.inst_param(repo_location, "repo_location", RepositoryLocation)
     check.inst_param(external_repo, "external_repo", ExternalRepository)
     check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
-    check.opt_dict_param(run_config, "run_config")
+    check.opt_dict_param(run_config, "run_config", key_type=str)
 
     check.opt_str_param(mode, "mode")
     check.opt_str_param(preset, "preset")
@@ -479,64 +534,48 @@ def _create_external_pipeline_run(
         solid_selection=solid_selection,
     )
 
-    subset_pipeline_result = repo_location.get_subset_external_pipeline_result(pipeline_selector)
-    if subset_pipeline_result.success == False:
-        raise DagsterLaunchFailedError(
-            "Failed to load external pipeline subset: {error_message}".format(
-                error_message=subset_pipeline_result.error.message
-            ),
-            serializable_error_info=subset_pipeline_result.error,
-        )
+    external_pipeline = repo_location.get_external_pipeline(pipeline_selector)
 
-    external_pipeline_subset = ExternalPipeline(
-        subset_pipeline_result.external_pipeline_data,
-        external_repo.handle,
-    )
-
-    pipeline_mode = mode or external_pipeline_subset.get_default_mode_name()
+    pipeline_mode = mode or external_pipeline.get_default_mode_name()
 
     external_execution_plan = repo_location.get_external_execution_plan(
-        external_pipeline_subset,
+        external_pipeline,
         run_config,
         pipeline_mode,
         step_keys_to_execute=None,
         known_state=None,
+        instance=instance,
     )
-    if isinstance(external_execution_plan, ExecutionPlanSnapshotErrorData):
-        raise DagsterLaunchFailedError(
-            "Failed to load external execution plan",
-            serializable_error_info=external_execution_plan.error,
-        )
-    else:
-        execution_plan_snapshot = external_execution_plan.execution_plan_snapshot
+    execution_plan_snapshot = external_execution_plan.execution_plan_snapshot
 
     return instance.create_run(
         pipeline_name=pipeline_name,
         run_id=run_id,
         run_config=run_config,
         mode=pipeline_mode,
-        solids_to_execute=external_pipeline_subset.solids_to_execute,
-        step_keys_to_execute=None,
+        solids_to_execute=external_pipeline.solids_to_execute,
+        step_keys_to_execute=execution_plan_snapshot.step_keys_to_execute,
         solid_selection=solid_selection,
         status=None,
         root_run_id=None,
         parent_run_id=None,
         tags=tags,
-        pipeline_snapshot=external_pipeline_subset.pipeline_snapshot,
+        pipeline_snapshot=external_pipeline.pipeline_snapshot,
         execution_plan_snapshot=execution_plan_snapshot,
-        parent_pipeline_snapshot=external_pipeline_subset.parent_pipeline_snapshot,
-        external_pipeline_origin=external_pipeline_subset.get_external_origin(),
+        parent_pipeline_snapshot=external_pipeline.parent_pipeline_snapshot,
+        external_pipeline_origin=external_pipeline.get_external_origin(),
+        pipeline_code_origin=external_pipeline.get_python_origin(),
     )
 
 
 def do_execute_command(
-    pipeline,
-    instance,
-    config,
-    mode=None,
-    tags=None,
-    solid_selection=None,
-    preset=None,
+    pipeline: IPipeline,
+    instance: DagsterInstance,
+    config: Optional[List[str]],
+    mode: Optional[str] = None,
+    tags: Optional[Dict[str, object]] = None,
+    solid_selection: Optional[List[str]] = None,
+    preset: Optional[str] = None,
 ):
     check.inst_param(pipeline, "pipeline", IPipeline)
     check.inst_param(instance, "instance", DagsterInstance)
@@ -554,18 +593,18 @@ def do_execute_command(
     )
 
 
-@click.command(
+@pipeline_cli.command(
     name="launch",
     help="Launch a pipeline using the run launcher configured on the Dagster instance.\n\n{instructions}".format(
         instructions=get_pipeline_instructions("launch")
     ),
 )
 @pipeline_target_argument
-@python_pipeline_config_argument("launch")
+@python_pipeline_or_job_config_argument("launch")
 @click.option(
     "--config-json",
     type=click.STRING,
-    help="JSON string of run config to use for this pipeline run. Cannot be used with -c / --config.",
+    help="JSON string of run config to use for this pipeline/job run. Cannot be used with -c / --config.",
 )
 @click.option(
     "--preset",
@@ -592,25 +631,30 @@ def do_execute_command(
         '   ancestors, "other_solid_a" itself, and "other_solid_b" and its direct child solids'
     ),
 )
-@click.option("--run-id", type=click.STRING, help="The ID to give to the launched pipeline run")
+@click.option("--run-id", type=click.STRING, help="The ID to give to the launched pipeline/job run")
 def pipeline_launch_command(**kwargs):
     with DagsterInstance.get() as instance:
         return execute_launch_command(instance, kwargs)
 
 
 @telemetry_wrapper
-def execute_launch_command(instance, kwargs):
-    preset = kwargs.get("preset")
-    mode = kwargs.get("mode")
+def execute_launch_command(
+    instance: DagsterInstance, kwargs: Dict[str, str], using_job_op_graph_apis: bool = False
+):
+    preset = cast(Optional[str], kwargs.get("preset"))
+    mode = cast(Optional[str], kwargs.get("mode"))
     check.inst_param(instance, "instance", DagsterInstance)
     config = get_config_from_args(kwargs)
 
-    with get_repository_location_from_kwargs(kwargs) as repo_location:
+    with get_workspace_from_kwargs(instance, version=dagster_version, kwargs=kwargs) as workspace:
+        repo_location = get_repository_location_from_workspace(workspace, kwargs.get("location"))
         external_repo = get_external_repository_from_repo_location(
-            repo_location, kwargs.get("repository")
+            repo_location, cast(Optional[str], kwargs.get("repository"))
         )
-        external_pipeline = get_external_pipeline_from_external_repo(
-            external_repo, kwargs.get("pipeline")
+        external_pipeline = get_external_pipeline_or_job_from_external_repo(
+            external_repo,
+            cast(Optional[str], kwargs.get("pipeline_or_job")),
+            using_job_op_graph_apis,
         )
 
         log_external_repo_stats(
@@ -637,13 +681,13 @@ def execute_launch_command(instance, kwargs):
             preset=preset,
             tags=run_tags,
             solid_selection=solid_selection,
-            run_id=kwargs.get("run_id"),
+            run_id=cast(Optional[str], kwargs.get("run_id")),
         )
 
-        return instance.submit_run(pipeline_run.run_id, external_pipeline)
+        return instance.submit_run(pipeline_run.run_id, workspace)
 
 
-@click.command(
+@pipeline_cli.command(
     name="scaffold_config",
     help="Scaffold the config for a pipeline.\n\n{instructions}".format(
         instructions=get_pipeline_in_same_python_env_instructions("scaffold_config")
@@ -655,14 +699,18 @@ def pipeline_scaffold_command(**kwargs):
     execute_scaffold_command(kwargs, click.echo)
 
 
-def execute_scaffold_command(cli_args, print_fn):
-    pipeline_origih = get_pipeline_python_origin_from_kwargs(cli_args)
-    pipeline = recon_pipeline_from_origin(pipeline_origih)
+def execute_scaffold_command(cli_args, print_fn, using_job_op_graph_apis=False):
+    pipeline_origin = get_pipeline_or_job_python_origin_from_kwargs(
+        cli_args, using_job_op_graph_apis
+    )
+    pipeline = recon_pipeline_from_origin(pipeline_origin)
     skip_non_required = cli_args["print_only_required"]
     do_scaffold_command(pipeline.get_definition(), print_fn, skip_non_required)
 
 
-def do_scaffold_command(pipeline_def, printer, skip_non_required):
+def do_scaffold_command(
+    pipeline_def: PipelineDefinition, printer: Callable[..., Any], skip_non_required: bool
+):
     check.inst_param(pipeline_def, "pipeline_def", PipelineDefinition)
     check.callable_param(printer, "printer")
     check.bool_param(skip_non_required, "skip_non_required")
@@ -694,7 +742,7 @@ def gen_partition_names_from_args(partition_names, kwargs):
         if len(selected_partitions) < len(selected_args):
             selected_names = [partition for partition in selected_partitions]
             unknown = [selected for selected in selected_args if selected not in selected_names]
-            raise click.UsageError("Unknown partitions: {}".format(unknown.join(", ")))
+            raise click.UsageError("Unknown partitions: {}".format(", ".join(unknown)))
         return selected_partitions
 
     start = validate_partition_slice(partition_names, "from", kwargs.get("from"))
@@ -703,23 +751,25 @@ def gen_partition_names_from_args(partition_names, kwargs):
     return partition_names[start:end]
 
 
-def get_config_from_args(kwargs):
-    config_files = kwargs.get("config")
+def get_config_from_args(kwargs: Dict[str, str]) -> Dict[str, object]:
+
+    config = kwargs.get("config")  # files
     config_json = kwargs.get("config_json")
 
-    if not config_files and not config_json:
+    if not config and not config_json:
         return {}
 
-    if config_files and config_json:
+    elif config and config_json:
         raise click.UsageError("Cannot specify both -c / --config and --config-json")
 
-    if config_files:
+    elif config:
         config_file_list = list(
-            check.opt_tuple_param(kwargs.get("config"), "config", default=(), of_type=str)
+            check.opt_tuple_param(config, "config", default=tuple(), of_type=str)
         )
         return get_run_config_from_file_list(config_file_list)
 
-    if config_json:
+    elif config_json:
+        config_json = cast(str, config_json)
         try:
             return json.loads(config_json)
 
@@ -730,6 +780,8 @@ def get_config_from_args(kwargs):
                     serializable_error_info_from_exc_info(sys.exc_info()).to_string(),
                 )
             )
+    else:
+        check.failed("Unhandled case getting config from kwargs")
 
 
 def get_tags_from_args(kwargs):
@@ -787,9 +839,9 @@ def validate_partition_slice(partition_names, name, value):
     return index if is_start else index + 1
 
 
-@click.command(
+@pipeline_cli.command(
     name="backfill",
-    help="Backfill a partitioned pipeline.\n\n{instructions}".format(
+    help="Backfill a partitioned pipeline/job.\n\n{instructions}".format(
         instructions=get_partitioned_pipeline_instructions("backfill")
     ),
 )
@@ -827,26 +879,32 @@ def validate_partition_slice(partition_names, name, value):
         "dagster pipeline backfill log_daily_stats --to 20191201"
     ),
 )
-@click.option("--tags", type=click.STRING, help="JSON string of tags to use for this pipeline run")
+@click.option(
+    "--tags", type=click.STRING, help="JSON string of tags to use for this pipeline/job run"
+)
 @click.option("--noprompt", is_flag=True)
 def pipeline_backfill_command(**kwargs):
     with DagsterInstance.get() as instance:
         execute_backfill_command(kwargs, click.echo, instance)
 
 
-def execute_backfill_command(cli_args, print_fn, instance):
-    with get_repository_location_from_kwargs(cli_args) as repo_location:
-        _execute_backfill_command_at_location(cli_args, print_fn, instance, repo_location)
+def execute_backfill_command(cli_args, print_fn, instance, using_graph_job_op_apis=False):
+    with get_workspace_from_kwargs(instance, version=dagster_version, kwargs=cli_args) as workspace:
+        repo_location = get_repository_location_from_workspace(workspace, cli_args.get("location"))
+        _execute_backfill_command_at_location(
+            cli_args, print_fn, instance, workspace, repo_location, using_graph_job_op_apis
+        )
 
 
-def _execute_backfill_command_at_location(cli_args, print_fn, instance, repo_location):
+def _execute_backfill_command_at_location(
+    cli_args, print_fn, instance, workspace, repo_location, using_graph_job_op_apis=False
+):
     external_repo = get_external_repository_from_repo_location(
         repo_location, cli_args.get("repository")
     )
 
-    external_pipeline = get_external_pipeline_from_external_repo(
-        external_repo,
-        cli_args.get("pipeline"),
+    external_pipeline = get_external_pipeline_or_job_from_external_repo(
+        external_repo, cli_args.get("pipeline_or_job")
     )
 
     noprompt = cli_args.get("noprompt")
@@ -859,7 +917,7 @@ def _execute_backfill_command_at_location(cli_args, print_fn, instance, repo_loc
 
     if not pipeline_partition_set_names:
         raise click.UsageError(
-            "No partition sets found for pipeline `{}`".format(external_pipeline.name)
+            "No partition sets found for pipeline/job `{}`".format(external_pipeline.name)
         )
     partition_set_name = cli_args.get("partition_set")
     if not partition_set_name:
@@ -883,22 +941,21 @@ def _execute_backfill_command_at_location(cli_args, print_fn, instance, repo_loc
 
     repo_handle = RepositoryHandle(
         repository_name=external_repo.name,
-        repository_location_handle=repo_location.location_handle,
+        repository_location=repo_location,
     )
 
-    # Resolve partitions to backfill
-    partition_names_or_error = repo_location.get_external_partition_names(
-        repo_handle,
-        partition_set_name,
-    )
-
-    if isinstance(partition_names_or_error, ExternalPartitionExecutionErrorData):
+    try:
+        partition_names_or_error = repo_location.get_external_partition_names(
+            repo_handle,
+            partition_set_name,
+        )
+    except Exception:
+        error_info = serializable_error_info_from_exc_info(sys.exc_info())
         raise DagsterBackfillFailedError(
-            "Failure fetching partition names for {partition_set_name}: {error_message}".format(
-                partition_set_name=partition_set_name,
-                error_message=partition_names_or_error.error.message,
+            "Failure fetching partition names: {error_message}".format(
+                error_message=error_info.message
             ),
-            serialized_error_info=partition_names_or_error.error,
+            serialized_error_info=error_info,
         )
 
     partition_names = gen_partition_names_from_args(
@@ -906,8 +963,9 @@ def _execute_backfill_command_at_location(cli_args, print_fn, instance, repo_loc
     )
 
     # Print backfill info
-    print_fn("\n     Pipeline: {}".format(external_pipeline.name))
-    print_fn("Partition set: {}".format(partition_set_name))
+    print_fn("\n Pipeline/Job: {}".format(external_pipeline.name))
+    if not using_graph_job_op_apis:
+        print_fn("Partition set: {}".format(partition_set_name))
     print_fn("   Partitions: {}\n".format(print_partition_format(partition_names, indent_level=15)))
 
     # Confirm and launch
@@ -928,19 +986,20 @@ def _execute_backfill_command_at_location(cli_args, print_fn, instance, repo_loc
             tags=run_tags,
             backfill_timestamp=pendulum.now("UTC").timestamp(),
         )
-        partition_execution_data = repo_location.get_external_partition_set_execution_param_data(
-            repository_handle=repo_handle,
-            partition_set_name=partition_set_name,
-            partition_names=partition_names,
-        )
-
-        if isinstance(partition_execution_data, ExternalPartitionExecutionErrorData):
-            instance.add_backfill(
-                backfill_job.with_status(BulkActionStatus.FAILED).with_error(
-                    partition_execution_data.error
+        try:
+            partition_execution_data = (
+                repo_location.get_external_partition_set_execution_param_data(
+                    repository_handle=repo_handle,
+                    partition_set_name=partition_set_name,
+                    partition_names=partition_names,
                 )
             )
-            return print_fn("Backfill failed: {}".format(partition_execution_data.error))
+        except Exception:
+            error_info = serializable_error_info_from_exc_info(sys.exc_info())
+            instance.add_backfill(
+                backfill_job.with_status(BulkActionStatus.FAILED).with_error(error_info)
+            )
+            return print_fn("Backfill failed: {}".format(error_info))
 
         assert isinstance(partition_execution_data, ExternalPartitionSetExecutionParamData)
 
@@ -954,7 +1013,7 @@ def _execute_backfill_command_at_location(cli_args, print_fn, instance, repo_loc
                 partition_data,
             )
             if pipeline_run:
-                instance.submit_run(pipeline_run.run_id, external_pipeline)
+                instance.submit_run(pipeline_run.run_id, workspace)
 
         instance.add_backfill(backfill_job.with_status(BulkActionStatus.COMPLETED))
 
@@ -962,6 +1021,3 @@ def _execute_backfill_command_at_location(cli_args, print_fn, instance, repo_loc
 
     else:
         print_fn("Aborted!")
-
-
-pipeline_cli = create_pipeline_cli_group()
