@@ -1,7 +1,9 @@
 import glob
 import os
+from typing import Optional, Sequence, Union
 
 import click
+
 from dagster import DagsterInvariantViolationError
 from dagster import __version__ as dagster_version
 from dagster import check
@@ -27,11 +29,11 @@ def print_changes(external_repository, instance, print_fn=print, preview=False):
     debug_info = instance.scheduler_debug_info()
     errors = debug_info.errors
     external_schedules = external_repository.get_external_schedules()
-    schedule_states = instance.all_stored_job_state(
+    schedule_states = instance.all_instigator_state(
         external_repository.get_external_origin_id(), InstigatorType.SCHEDULE
     )
     external_schedules_dict = {s.get_external_origin_id(): s for s in external_schedules}
-    schedule_states_dict = {s.job_origin_id: s for s in schedule_states}
+    schedule_states_dict = {s.instigator_origin_id: s for s in schedule_states}
 
     external_schedule_origin_ids = set(external_schedules_dict.keys())
     schedule_state_ids = set(schedule_states_dict.keys())
@@ -43,7 +45,7 @@ def print_changes(external_repository, instance, print_fn=print, preview=False):
     for schedule_origin_id in external_schedule_origin_ids & schedule_state_ids:
         schedule_state = schedule_states_dict[schedule_origin_id]
         external_schedule = external_schedules_dict[schedule_origin_id]
-        if schedule_state.job_specific_data.cron_schedule != external_schedule.cron_schedule:
+        if schedule_state.instigator_data.cron_schedule != external_schedule.cron_schedule:
             changed_schedules.append(schedule_origin_id)
 
     if not errors and not added_schedules and not changed_schedules and not removed_schedules:
@@ -94,7 +96,7 @@ def print_changes(external_repository, instance, print_fn=print, preview=False):
         )
         print_fn(
             click.style("\t cron_schedule: ", fg="yellow")
-            + click.style(schedule_state.job_specific_data.cron_schedule, fg="red")
+            + click.style(schedule_state.instigator_data.cron_schedule, fg="red")
             + " => "
             + click.style(external_schedule.cron_schedule, fg="green")
         )
@@ -103,14 +105,15 @@ def print_changes(external_repository, instance, print_fn=print, preview=False):
         print_fn(
             click.style(
                 "  - {name} (delete) [{id}]".format(
-                    name=schedule_states_dict[schedule_origin_id].job_name, id=schedule_origin_id
+                    name=schedule_states_dict[schedule_origin_id].instigator_name,
+                    id=schedule_origin_id,
                 ),
                 fg="red",
             )
         )
 
 
-def check_repo_and_scheduler(repository, instance):
+def check_repo_and_scheduler(repository: ExternalRepository, instance: DagsterInstance) -> None:
     check.inst_param(repository, "repository", ExternalRepository)
     check.inst_param(instance, "instance", DagsterInstance)
 
@@ -182,36 +185,30 @@ def execute_list_command(running_filter, stopped_filter, name_filter, cli_args,
 
             repo_schedules = external_repo.get_external_schedules()
             stored_schedules_by_origin_id = {
-                stored_schedule_state.job_origin_id: stored_schedule_state
-                for stored_schedule_state in instance.all_stored_job_state(
-                    external_repo.get_external_origin_id(), job_type=InstigatorType.SCHEDULE
+                stored_schedule_state.instigator_origin_id: stored_schedule_state
+                for stored_schedule_state in instance.all_instigator_state(
+                    external_repo.get_external_origin_id(), instigator_type=InstigatorType.SCHEDULE
                 )
             }
 
             first = True
 
             for external_schedule in repo_schedules:
-                stored_schedule_state = stored_schedules_by_origin_id.get(
-                    external_schedule.get_external_origin_id()
+                schedule_state = external_schedule.get_current_instigator_state(
+                    stored_schedules_by_origin_id.get(external_schedule.get_external_origin_id())
                 )
-                if running_filter and (
-                    not stored_schedule_state
-                    or stored_schedule_state.status == InstigatorStatus.STOPPED
-                ):
+
+                if running_filter and not schedule_state.is_running:
                     continue
-                if stopped_filter and stored_schedule_state and InstigatorStatus.RUNNING:
+                if stopped_filter and schedule_state.is_running:
                     continue
 
                 if name_filter:
                     print_fn(external_schedule.name)
                     continue
 
-                status = (
-                    stored_schedule_state.status
-                    if stored_schedule_state
-                    else InstigatorStatus.STOPPED
-                )
-                schedule_title = f"Schedule: {external_schedule.name} [{status.value}]"
+                status = "RUNNING" if schedule_state.is_running else "STOPPED"
+                schedule_title = f"Schedule: {external_schedule.name} [{status}]"
                 if not first:
                     print_fn("*" * len(schedule_title))
 
@@ -221,7 +218,7 @@ def execute_list_command(running_filter, stopped_filter, name_filter, cli_args,
                 print_fn(f"Cron Schedule: {external_schedule.cron_schedule}")
 
 
-def extract_schedule_name(schedule_name):
+def extract_schedule_name(schedule_name: Optional[Union[str, Sequence[str]]]) -> Optional[str]:
     if schedule_name and not isinstance(schedule_name, str):
         if len(schedule_name) == 1:
             return schedule_name[0]
@@ -231,6 +228,7 @@ def extract_schedule_name(schedule_name):
                     schedule_name=repr(schedule_name)
                 )
             )
+    return None
 
 
 @schedule_cli.command(name="start", help="Start an existing schedule.")
@@ -260,7 +258,7 @@ def execute_start_command(schedule_name, all_flag, cli_args, print_fn):
             if all_flag:
                 for external_schedule in external_repo.get_external_schedules():
                     try:
-                        instance.start_schedule_and_update_storage_state(external_schedule)
+                        instance.start_schedule(external_schedule)
                     except DagsterInvariantViolationError as ex:
                         raise click.UsageError(ex)
 
@@ -272,9 +270,7 @@ def execute_start_command(schedule_name, all_flag, cli_args, print_fn):
             else:
                 try:
 
-                    instance.start_schedule_and_update_storage_state(
-                        external_repo.get_external_schedule(schedule_name)
-                    )
+                    instance.start_schedule(external_repo.get_external_schedule(schedule_name))
                 except DagsterInvariantViolationError as ex:
                     raise click.UsageError(ex)
 
@@ -297,8 +293,9 @@ def execute_stop_command(schedule_name, cli_args, print_fn, instance=None):
             check_repo_and_scheduler(external_repo, instance)
 
             try:
-                instance.stop_schedule_and_update_storage_state(
-                    external_repo.get_external_schedule(schedule_name).get_external_origin_id()
+                external_schedule = external_repo.get_external_schedule(schedule_name)
+                instance.stop_schedule(
+                    external_schedule.get_external_origin_id(), external_schedule
                 )
             except DagsterInvariantViolationError as ex:
                 raise click.UsageError(ex)
@@ -396,18 +393,19 @@ def execute_restart_command(schedule_name, all_running_flag, cli_args, print_fn)
             repository_name = external_repo.name
 
             if all_running_flag:
-                for schedule_state in instance.all_stored_job_state(
+                for schedule_state in instance.all_instigator_state(
                     external_repo.get_external_origin_id(), InstigatorType.SCHEDULE
                 ):
                     if schedule_state.status == InstigatorStatus.RUNNING:
                         try:
                             external_schedule = external_repo.get_external_schedule(
-                                schedule_state.job_name
+                                schedule_state.instigator_name
                             )
-                            instance.stop_schedule_and_update_storage_state(
-                                schedule_state.job_origin_id
+                            instance.stop_schedule(
+                                schedule_state.instigator_origin_id,
+                                external_schedule,
                             )
-                            instance.start_schedule_and_update_storage_state(external_schedule)
+                            instance.start_schedule(external_schedule)
                         except DagsterInvariantViolationError as ex:
                             raise click.UsageError(ex)
 
@@ -418,17 +416,19 @@ def execute_restart_command(schedule_name, all_running_flag, cli_args, print_fn)
                 )
             else:
                 external_schedule = external_repo.get_external_schedule(schedule_name)
-                schedule_state = instance.get_job_state(external_schedule.get_external_origin_id())
+                schedule_state = instance.get_instigator_state(
+                    external_schedule.get_external_origin_id()
+                )
                 if schedule_state != None and schedule_state.status != InstigatorStatus.RUNNING:
                     click.UsageError(
                         "Cannot restart a schedule {name} because is not currently running".format(
-                            name=schedule_state.job_name
+                            name=schedule_state.instigator_name
                         )
                     )
 
                 try:
-                    instance.stop_schedule_and_update_storage_state(schedule_state.job_origin_id)
-                    instance.start_schedule_and_update_storage_state(external_schedule)
+                    instance.stop_schedule(schedule_state.instigator_origin_id, external_schedule)
+                    instance.start_schedule(external_schedule)
                 except DagsterInvariantViolationError as ex:
                     raise click.UsageError(ex)
 
