@@ -1,8 +1,10 @@
-from typing import AbstractSet, Any, Dict, List, Mapping, Optional, Sequence, Tuple, Union, cast
+import itertools
+import warnings
+from typing import AbstractSet, Any, Dict, Mapping, Optional, Sequence, Tuple, Union, cast
 
 from dagster import check
 from dagster.core.definitions.config import ConfigMapping
-from dagster.core.definitions.decorators.op import op
+from dagster.core.definitions.decorators.op_decorator import op
 from dagster.core.definitions.dependency import (
     DependencyDefinition,
     IDependencyDefinition,
@@ -22,22 +24,22 @@ from dagster.core.execution.context.input import InputContext, build_input_conte
 from dagster.core.execution.context.output import build_output_context
 from dagster.core.storage.fs_asset_io_manager import fs_asset_io_manager
 from dagster.core.storage.root_input_manager import RootInputManagerDefinition, root_input_manager
-from dagster.utils.backcompat import experimental
+from dagster.utils.backcompat import ExperimentalWarning, experimental
 from dagster.utils.merger import merge_dicts
 
-from .asset import AssetsDefinition
 from .asset_partitions import get_upstream_partitions_for_partition_range
+from .assets import AssetsDefinition
 from .source_asset import SourceAsset
 
 
 @experimental
 def build_assets_job(
     name: str,
-    assets: List[AssetsDefinition],
+    assets: Sequence[AssetsDefinition],
     source_assets: Optional[Sequence[Union[SourceAsset, AssetsDefinition]]] = None,
-    resource_defs: Optional[Dict[str, ResourceDefinition]] = None,
+    resource_defs: Optional[Mapping[str, ResourceDefinition]] = None,
     description: Optional[str] = None,
-    config: Union[ConfigMapping, Dict[str, Any], PartitionedConfig] = None,
+    config: Optional[Union[ConfigMapping, Dict[str, Any], PartitionedConfig]] = None,
     tags: Optional[Dict[str, Any]] = None,
     executor_def: Optional[ExecutorDefinition] = None,
 ) -> JobDefinition:
@@ -75,14 +77,16 @@ def build_assets_job(
         JobDefinition: A job that materializes the given assets.
     """
     check.str_param(name, "name")
-    check.list_param(assets, "assets", of_type=AssetsDefinition)
-    check.opt_list_param(source_assets, "source_assets", of_type=(SourceAsset, AssetsDefinition))
+    check.sequence_param(assets, "assets", of_type=AssetsDefinition)
+    check.opt_sequence_param(
+        source_assets, "source_assets", of_type=(SourceAsset, AssetsDefinition)
+    )
     check.opt_str_param(description, "description")
     source_assets_by_key = build_source_assets_by_key(source_assets)
 
     op_defs = build_op_deps(assets, source_assets_by_key.keys())
     root_manager = build_root_manager(source_assets_by_key)
-    partitioned_config = build_job_partitions_from_assets(assets)
+    partitioned_config = build_job_partitions_from_assets(assets, source_assets or [])
 
     return GraphDefinition(
         name=name,
@@ -104,13 +108,14 @@ def build_assets_job(
 
 def build_job_partitions_from_assets(
     assets: Sequence[AssetsDefinition],
+    source_assets: Sequence[Union[SourceAsset, AssetsDefinition]],
 ) -> Optional[PartitionedConfig]:
     assets_with_partitions_defs = [assets_def for assets_def in assets if assets_def.partitions_def]
 
     if len(assets_with_partitions_defs) == 0:
         return None
 
-    first_assets_with_partitions_def = assets_with_partitions_defs[0]
+    first_assets_with_partitions_def: AssetsDefinition = assets_with_partitions_defs[0]
     for assets_def in assets_with_partitions_defs:
         if assets_def.partitions_def != first_assets_with_partitions_def.partitions_def:
             first_asset_key = next(iter(assets_def.asset_keys)).to_string()
@@ -121,9 +126,14 @@ def build_job_partitions_from_assets(
                 f"'{second_asset_key}' have different partitions definitions. "
             )
 
-    assets_defs_by_asset_key = {
-        asset_key: assets_def for assets_def in assets for asset_key in assets_def.asset_keys
-    }
+    partitions_defs_by_asset_key: Dict[AssetKey, PartitionsDefinition] = {}
+    asset: Union[AssetsDefinition, SourceAsset]
+    for asset in itertools.chain.from_iterable([assets, source_assets]):
+        if isinstance(asset, AssetsDefinition) and asset.partitions_def is not None:
+            for asset_key in asset.asset_keys:
+                partitions_defs_by_asset_key[asset_key] = asset.partitions_def
+        elif isinstance(asset, SourceAsset) and asset.partitions_def is not None:
+            partitions_defs_by_asset_key[asset.key] = asset.partitions_def
 
     def asset_partitions_for_job_partition(
         job_partition_key: str,
@@ -151,13 +161,10 @@ def build_job_partitions_from_assets(
 
             inputs_dict: Dict[str, Dict[str, Any]] = {}
             for in_asset_key, input_def in assets_def.input_defs_by_asset_key.items():
-                upstream_assets_def = assets_defs_by_asset_key[in_asset_key]
-                if (
-                    assets_def.partitions_def is not None
-                    and upstream_assets_def.partitions_def is not None
-                ):
+                upstream_partitions_def = partitions_defs_by_asset_key.get(in_asset_key)
+                if assets_def.partitions_def is not None and upstream_partitions_def is not None:
                     upstream_partition_key_range = get_upstream_partitions_for_partition_range(
-                        assets_def, upstream_assets_def, in_asset_key, asset_partition_key_range
+                        assets_def, upstream_partitions_def, in_asset_key, asset_partition_key_range
                     )
                     inputs_dict[input_def.name] = {
                         "start": upstream_partition_key_range.start,
@@ -197,7 +204,7 @@ def build_source_assets_by_key(
 
 
 def build_op_deps(
-    multi_asset_defs: List[AssetsDefinition], source_paths: AbstractSet[AssetKey]
+    multi_asset_defs: Sequence[AssetsDefinition], source_paths: AbstractSet[AssetKey]
 ) -> Dict[Union[str, NodeInvocation], Dict[str, IDependencyDefinition]]:
     op_outputs_by_asset: Dict[AssetKey, Tuple[OpDefinition, str]] = {}
     for multi_asset_def in multi_asset_defs:
@@ -233,32 +240,43 @@ def build_root_manager(
     source_asset_io_manager_keys = {
         source_asset.io_manager_key for source_asset in source_assets_by_key.values()
     }
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", category=ExperimentalWarning)
 
-    @root_input_manager(required_resource_keys=source_asset_io_manager_keys)
-    def _root_manager(input_context: InputContext) -> Any:
-        source_asset_key = cast(AssetKey, input_context.asset_key)
-        source_asset = source_assets_by_key[source_asset_key]
+        @root_input_manager(required_resource_keys=source_asset_io_manager_keys)
+        def _root_manager(input_context: InputContext) -> Any:
+            source_asset_key = cast(AssetKey, input_context.asset_key)
+            source_asset = source_assets_by_key[source_asset_key]
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore", category=ExperimentalWarning)
 
-        @op(out={source_asset_key.path[-1]: Out(asset_key=source_asset_key)})
-        def _op():
-            pass
+                @op(out={source_asset_key.path[-1]: Out(asset_key=source_asset_key)})
+                def _op():
+                    pass
 
-        output_context = build_output_context(
-            name=source_asset_key.path[-1],
-            step_key="none",
-            solid_def=_op,
-            metadata=source_asset.metadata,
-        )
-        input_context_with_upstream = build_input_context(
-            name=input_context.name,
-            metadata=input_context.metadata,
-            config=input_context.config,
-            dagster_type=input_context.dagster_type,
-            upstream_output=output_context,
-            op_def=input_context.op_def,
-        )
+            resource_config = input_context.step_context.resolved_run_config.resources[
+                source_asset.io_manager_key
+            ].config
 
-        io_manager = getattr(cast(Any, input_context.resources), source_asset.io_manager_key)
-        return io_manager.load_input(input_context_with_upstream)
+            output_context = build_output_context(
+                name=source_asset_key.path[-1],
+                step_key="none",
+                solid_def=_op,
+                metadata=cast(Dict[str, Any], source_asset.metadata),
+                resource_config=resource_config,
+            )
+            input_context_with_upstream = build_input_context(
+                name=input_context.name,
+                metadata=input_context.metadata,
+                config=input_context.config,
+                dagster_type=input_context.dagster_type,
+                upstream_output=output_context,
+                op_def=input_context.op_def,
+                step_context=input_context.step_context,
+                resource_config=resource_config,
+            )
+
+            io_manager = getattr(cast(Any, input_context.resources), source_asset.io_manager_key)
+            return io_manager.load_input(input_context_with_upstream)
 
     return _root_manager
