@@ -1,9 +1,26 @@
+import warnings
 from abc import ABC, abstractmethod
-from typing import Callable, Dict, Generic, List, Optional, Type, TypeVar, Union, cast
+from types import FunctionType
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Generic,
+    List,
+    Mapping,
+    Optional,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+)
 
 from dagster import check
+from dagster.core.asset_defs.source_asset import SourceAsset
 from dagster.core.errors import DagsterInvalidDefinitionError, DagsterInvariantViolationError
 from dagster.utils import merge_dicts
+from dagster.utils.backcompat import ExperimentalWarning
 
 from .events import AssetKey
 from .graph_definition import GraphDefinition, SubselectedGraphDefinition
@@ -14,6 +31,9 @@ from .schedule_definition import ScheduleDefinition
 from .sensor_definition import SensorDefinition
 from .utils import check_valid_name
 
+if TYPE_CHECKING:
+    from dagster.core.asset_defs.asset_group import AssetGroup
+
 VALID_REPOSITORY_DATA_DICT_KEYS = {
     "pipelines",
     "partition_sets",
@@ -25,6 +45,7 @@ VALID_REPOSITORY_DATA_DICT_KEYS = {
 RepositoryLevelDefinition = TypeVar(
     "RepositoryLevelDefinition",
     PipelineDefinition,
+    JobDefinition,
     PartitionSetDefinition,
     ScheduleDefinition,
     SensorDefinition,
@@ -37,7 +58,7 @@ class _CacheingDefinitionIndex(Generic[RepositoryLevelDefinition]):
         definition_class: Type[RepositoryLevelDefinition],
         definition_class_name: str,
         definition_kind: str,
-        definitions: Dict[
+        definitions: Mapping[
             str, Union[RepositoryLevelDefinition, Callable[[], RepositoryLevelDefinition]]
         ],
         validation_fn: Callable[[RepositoryLevelDefinition], RepositoryLevelDefinition],
@@ -71,20 +92,20 @@ class _CacheingDefinitionIndex(Generic[RepositoryLevelDefinition]):
             [RepositoryLevelDefinition], RepositoryLevelDefinition
         ] = validation_fn
 
-        self._definitions: Dict[
+        self._definitions: Mapping[
             str, Union[RepositoryLevelDefinition, Callable[[], RepositoryLevelDefinition]]
         ] = definitions
         self._definition_cache: Dict[str, RepositoryLevelDefinition] = {}
         self._definition_names: Optional[List[str]] = None
 
-        self._lazy_definitions_fn: Optional[
-            Callable[[], List[RepositoryLevelDefinition]]
+        self._lazy_definitions_fn: Callable[
+            [], List[RepositoryLevelDefinition]
         ] = lazy_definitions_fn or (lambda: [])
         self._lazy_definitions: Optional[List[RepositoryLevelDefinition]] = None
 
         self._all_definitions: Optional[List[RepositoryLevelDefinition]] = None
 
-    def _get_lazy_definitions(self):
+    def _get_lazy_definitions(self) -> List[RepositoryLevelDefinition]:
         if self._lazy_definitions is None:
             self._lazy_definitions = self._lazy_definitions_fn()
             for definition in self._lazy_definitions:
@@ -193,14 +214,14 @@ class RepositoryData(ABC):
     """
 
     @abstractmethod
-    def get_all_pipelines(self):
+    def get_all_pipelines(self) -> List[PipelineDefinition]:
         """Return all pipelines/jobs in the repository as a list.
 
         Returns:
             List[PipelineDefinition]: All pipelines/jobs in the repository.
         """
 
-    def get_all_jobs(self):
+    def get_all_jobs(self) -> List[JobDefinition]:
         """Return all jobs in the repository as a list.
 
         Returns:
@@ -208,7 +229,7 @@ class RepositoryData(ABC):
         """
         return [job for job in self.get_all_pipelines() if isinstance(job, JobDefinition)]
 
-    def get_pipeline_names(self):
+    def get_pipeline_names(self) -> List[str]:
         """Get the names of all pipelines/jobs in the repository.
 
         Returns:
@@ -216,7 +237,7 @@ class RepositoryData(ABC):
         """
         return [pipeline_def.name for pipeline_def in self.get_all_pipelines()]
 
-    def get_job_names(self):
+    def get_job_names(self) -> List[str]:
         """Get the names of all jobs in the repository.
 
         Returns:
@@ -224,7 +245,7 @@ class RepositoryData(ABC):
         """
         return [job_def.name for job_def in self.get_all_jobs()]
 
-    def has_pipeline(self, pipeline_name):
+    def has_pipeline(self, pipeline_name: str) -> bool:
         """Check if a pipeline/job with a given name is present in the repository.
 
         Args:
@@ -235,7 +256,7 @@ class RepositoryData(ABC):
         """
         return pipeline_name in self.get_pipeline_names()
 
-    def has_job(self, job_name):
+    def has_job(self, job_name: str) -> bool:
         """Check if a job with a given name is present in the repository.
 
         Args:
@@ -246,7 +267,7 @@ class RepositoryData(ABC):
         """
         return job_name in self.get_job_names()
 
-    def get_pipeline(self, pipeline_name):
+    def get_pipeline(self, pipeline_name) -> PipelineDefinition:
         """Get a pipeline/job by name.
 
         Args:
@@ -264,7 +285,7 @@ class RepositoryData(ABC):
             )
         return pipelines_with_name[0]
 
-    def get_job(self, job_name):
+    def get_job(self, job_name: str) -> JobDefinition:
         """Get a job by name.
 
         Args:
@@ -286,7 +307,7 @@ class RepositoryData(ABC):
         """
         return [partition_set.name for partition_set in self.get_all_partition_sets()]
 
-    def has_partition_set(self, partition_set_name):
+    def has_partition_set(self, partition_set_name: str) -> bool:
         """Check if a partition set with a given name is present in the repository.
 
         Args:
@@ -297,7 +318,7 @@ class RepositoryData(ABC):
         """
         return partition_set_name in self.get_partition_set_names()
 
-    def get_all_partition_sets(self):
+    def get_all_partition_sets(self) -> List[PartitionSetDefinition]:
         """Return all partition sets in the repository as a list.
 
         Returns:
@@ -305,7 +326,7 @@ class RepositoryData(ABC):
         """
         return []
 
-    def get_partition_set(self, partition_set_name):
+    def get_partition_set(self, partition_set_name: str) -> PartitionSetDefinition:
         """Get a partition set by name.
 
         Args:
@@ -325,7 +346,7 @@ class RepositoryData(ABC):
             )
         return partition_sets_with_name[0]
 
-    def get_schedule_names(self):
+    def get_schedule_names(self) -> List[str]:
         """Get the names of all schedules in the repository.
 
         Returns:
@@ -333,7 +354,7 @@ class RepositoryData(ABC):
         """
         return [schedule.name for schedule in self.get_all_schedules()]
 
-    def get_all_schedules(self):
+    def get_all_schedules(self) -> List[ScheduleDefinition]:
         """Return all schedules in the repository as a list.
 
         Returns:
@@ -341,7 +362,7 @@ class RepositoryData(ABC):
         """
         return []
 
-    def get_schedule(self, schedule_name):
+    def get_schedule(self, schedule_name: str) -> ScheduleDefinition:
         """Get a schedule by name.
 
         args:
@@ -359,16 +380,16 @@ class RepositoryData(ABC):
             )
         return schedules_with_name[0]
 
-    def has_schedule(self, schedule_name):
+    def has_schedule(self, schedule_name: str) -> bool:
         return schedule_name in self.get_schedule_names()
 
-    def get_all_sensors(self):
+    def get_all_sensors(self) -> List[SensorDefinition]:
         return []
 
-    def get_sensor_names(self):
+    def get_sensor_names(self) -> List[str]:
         return [sensor.name for sensor in self.get_all_sensors()]
 
-    def get_sensor(self, sensor_name):
+    def get_sensor(self, sensor_name: str) -> SensorDefinition:
         sensors_with_name = [
             sensor for sensor in self.get_all_sensors() if sensor.name == sensor_name
         ]
@@ -378,17 +399,34 @@ class RepositoryData(ABC):
             )
         return sensors_with_name[0]
 
-    def has_sensor(self, sensor_name):
+    def has_sensor(self, sensor_name: str) -> bool:
         return sensor_name in self.get_sensor_names()
 
-    def get_source_assets_by_key(self):
+    def get_source_assets_by_key(self) -> Mapping[AssetKey, SourceAsset]:
         return {}
 
 
+T = TypeVar("T")
+Resolvable = Callable[[], T]
+
+
 class CachingRepositoryData(RepositoryData):
     """Default implementation of RepositoryData used by the :py:func:`@repository <repository>` decorator."""
 
-    def __init__(self, pipelines, jobs, partition_sets, schedules, sensors, source_assets):
+    _all_jobs: Optional[List[JobDefinition]]
+    _all_pipelines: Optional[List[PipelineDefinition]]
+
+    def __init__(
+        self,
+        pipelines: Mapping[str, Union[PipelineDefinition, Resolvable[PipelineDefinition]]],
+        jobs: Mapping[str, Union[JobDefinition, Resolvable[JobDefinition]]],
+        partition_sets: Mapping[
+            str, Union[PartitionSetDefinition, Resolvable[PartitionSetDefinition]]
+        ],
+        schedules: Mapping[str, Union[ScheduleDefinition, Resolvable[ScheduleDefinition]]],
+        sensors: Mapping[str, Union[SensorDefinition, Resolvable[SensorDefinition]]],
+        source_assets: Mapping[AssetKey, SourceAsset],
+    ):
         """Constructs a new CachingRepositoryData object.
 
         You may pass pipeline, job, partition_set, and schedule definitions directly, or you may pass
@@ -401,24 +439,37 @@ class CachingRepositoryData(RepositoryData):
         at retrieval time.
 
         Args:
-            pipelines (Dict[str, Union[PipelineDefinition, Callable[[], PipelineDefinition]]]):
+            pipelines (Mapping[str, Union[PipelineDefinition, Callable[[], PipelineDefinition]]]):
                 The pipeline definitions belonging to the repository.
-            jobs (Dict[str, Union[JobDefinition, Callable[[], JobDefinition]]]):
+            jobs (Mapping[str, Union[JobDefinition, Callable[[], JobDefinition]]]):
                 The job definitions belonging to the repository.
-            partition_sets (Dict[str, Union[PartitionSetDefinition, Callable[[], PartitionSetDefinition]]]):
+            partition_sets (Mapping[str, Union[PartitionSetDefinition, Callable[[], PartitionSetDefinition]]]):
                 The partition sets belonging to the repository.
-            schedules (Dict[str, Union[ScheduleDefinition, Callable[[], ScheduleDefinition]]]):
+            schedules (Mapping[str, Union[ScheduleDefinition, Callable[[], ScheduleDefinition]]]):
                 The schedules belonging to the repository.
-            sensors (Dict[str, Union[SensorDefinition, Callable[[], SensorDefinition]]]):
+            sensors (Mapping[str, Union[SensorDefinition, Callable[[], SensorDefinition]]]):
                 The sensors belonging to a repository.
-            source_assets (Dict[str, SourceAsset]): The source assets belonging to a repository.
+            source_assets (Mapping[AssetKey, SourceAsset]): The source assets belonging to a repository.
         """
-        check.dict_param(pipelines, "pipelines", key_type=str)
-        check.dict_param(jobs, "jobs", key_type=str)
-        check.dict_param(partition_sets, "partition_sets", key_type=str)
-        check.dict_param(schedules, "schedules", key_type=str)
-        check.dict_param(sensors, "sensors", key_type=str)
-        check.dict_param(source_assets, "source_assets", key_type=AssetKey)
+        check.mapping_param(
+            pipelines, "pipelines", key_type=str, value_type=(PipelineDefinition, FunctionType)
+        )
+        check.mapping_param(jobs, "jobs", key_type=str, value_type=(JobDefinition, FunctionType))
+        check.mapping_param(
+            partition_sets,
+            "partition_sets",
+            key_type=str,
+            value_type=(PartitionSetDefinition, FunctionType),
+        )
+        check.mapping_param(
+            schedules, "schedules", key_type=str, value_type=(ScheduleDefinition, FunctionType)
+        )
+        check.mapping_param(
+            sensors, "sensors", key_type=str, value_type=(SensorDefinition, FunctionType)
+        )
+        check.mapping_param(
+            source_assets, "source_assets", key_type=AssetKey, value_type=SourceAsset
+        )
 
         self._pipelines = _CacheingDefinitionIndex(
             PipelineDefinition,
@@ -450,7 +501,7 @@ class CachingRepositoryData(RepositoryData):
         ]
         self._source_assets = source_assets
 
-        def load_partition_sets_from_pipelines():
+        def load_partition_sets_from_pipelines() -> List[PartitionSetDefinition]:
             job_partition_sets = []
             for pipeline in self.get_all_pipelines():
                 if isinstance(pipeline, JobDefinition):
@@ -488,7 +539,7 @@ class CachingRepositoryData(RepositoryData):
         self._all_jobs = None
 
     @staticmethod
-    def from_dict(repository_definitions):
+    def from_dict(repository_definitions: Dict[str, Dict[str, Any]]) -> "CachingRepositoryData":
         """Static constructor.
 
         Args:
@@ -531,7 +582,7 @@ class CachingRepositoryData(RepositoryData):
         )
         if duplicate_keys:
             raise DagsterInvalidDefinitionError(
-                f"Duplicate definitions between schedules and sensors found for keys: {duplicate_keys.join(', ')}"
+                f"Duplicate definitions between schedules and sensors found for keys: {', '.join(duplicate_keys)}"
             )
 
         # merge jobs in to pipelines while they are just implemented as pipelines
@@ -551,21 +602,33 @@ class CachingRepositoryData(RepositoryData):
         return CachingRepositoryData(**repository_definitions, source_assets={})
 
     @classmethod
-    def from_list(cls, repository_definitions):
+    def from_list(
+        cls,
+        repository_definitions: List[
+            Union[
+                PipelineDefinition,
+                PartitionSetDefinition,
+                ScheduleDefinition,
+                SensorDefinition,
+                "AssetGroup",
+                GraphDefinition,
+            ]
+        ],
+    ) -> "CachingRepositoryData":
         """Static constructor.
 
         Args:
-            repository_definitions (List[Union[PipelineDefinition, PartitionSetDefinition, ScheduleDefinition, AssetGroup]]):
+            repository_definitions (List[Union[PipelineDefinition, PartitionSetDefinition, ScheduleDefinition, SensorDefinition, AssetGroup, GraphDefinition]]):
                 Use this constructor when you have no need to lazy load pipelines/jobs or other
                 definitions.
         """
         from dagster.core.asset_defs import AssetGroup, build_assets_job
 
-        pipelines_or_jobs = {}
-        partition_sets = {}
-        schedules = {}
-        sensors = {}
-        source_assets = {}
+        pipelines_or_jobs: Dict[str, Union[PipelineDefinition, JobDefinition]] = {}
+        partition_sets: Dict[str, PartitionSetDefinition] = {}
+        schedules: Dict[str, ScheduleDefinition] = {}
+        sensors: Dict[str, SensorDefinition] = {}
+        source_assets: Dict[AssetKey, SourceAsset] = {}
         for definition in repository_definitions:
             if isinstance(definition, PipelineDefinition):
                 if (
@@ -577,6 +640,10 @@ class CachingRepositoryData(RepositoryData):
                             target_type=definition.target_type, target=definition.describe_target()
                         )
                     )
+                if definition.name == AssetGroup.all_assets_job_name():
+                    raise DagsterInvalidDefinitionError(
+                        f"Attempted to provide job called {AssetGroup.all_assets_job_name()} to repository, which is a reserved name. Please rename the job."
+                    )
                 pipelines_or_jobs[definition.name] = definition
             elif isinstance(definition, PartitionSetDefinition):
                 if definition.name in partition_sets:
@@ -627,13 +694,20 @@ class CachingRepositoryData(RepositoryData):
 
             elif isinstance(definition, AssetGroup):
                 asset_group = definition
-                pipelines_or_jobs[asset_group.all_assets_job_name] = build_assets_job(
-                    asset_group.all_assets_job_name,
-                    assets=asset_group.assets,
-                    source_assets=asset_group.source_assets,
-                    resource_defs=asset_group.resource_defs,
-                    executor_def=asset_group.executor_def,
-                )
+
+                if asset_group.all_assets_job_name() in pipelines_or_jobs:
+                    raise DagsterInvalidDefinitionError(
+                        "When constructing repository, attempted to pass multiple AssetGroups. There can only be one AssetGroup per repository."
+                    )
+                with warnings.catch_warnings():
+                    warnings.simplefilter("ignore", category=ExperimentalWarning)
+                    pipelines_or_jobs[asset_group.all_assets_job_name()] = build_assets_job(
+                        asset_group.all_assets_job_name(),
+                        assets=asset_group.assets,
+                        source_assets=asset_group.source_assets,
+                        resource_defs=asset_group.resource_defs,
+                        executor_def=asset_group.executor_def,
+                    )
                 source_assets = {
                     source_asset.key: source_asset for source_asset in asset_group.source_assets
                 }
@@ -641,8 +715,8 @@ class CachingRepositoryData(RepositoryData):
             else:
                 check.failed(f"Unexpected repository entry {definition}")
 
-        pipelines = {}
-        jobs = {}
+        pipelines: Dict[str, PipelineDefinition] = {}
+        jobs: Dict[str, JobDefinition] = {}
         for name, pipeline_or_job in pipelines_or_jobs.items():
             if isinstance(pipeline_or_job, JobDefinition):
                 jobs[name] = pipeline_or_job
@@ -658,7 +732,7 @@ class CachingRepositoryData(RepositoryData):
             source_assets=source_assets,
         )
 
-    def get_pipeline_names(self):
+    def get_pipeline_names(self) -> List[str]:
         """Get the names of all pipelines/jobs in the repository.
 
         Returns:
@@ -666,7 +740,7 @@ class CachingRepositoryData(RepositoryData):
         """
         return self._pipelines.get_definition_names() + self.get_job_names()
 
-    def get_job_names(self):
+    def get_job_names(self) -> List[str]:
         """Get the names of all jobs in the repository.
 
         Returns:
@@ -674,7 +748,7 @@ class CachingRepositoryData(RepositoryData):
         """
         return self._jobs.get_definition_names()
 
-    def has_pipeline(self, pipeline_name):
+    def has_pipeline(self, pipeline_name: str) -> bool:
         """Check if a pipeline/job with a given name is present in the repository.
 
         Args:
@@ -689,7 +763,7 @@ class CachingRepositoryData(RepositoryData):
             pipeline_name
         )
 
-    def has_job(self, job_name):
+    def has_job(self, job_name: str) -> bool:
         """Check if a job with a given name is present in the repository.
 
         Args:
@@ -701,7 +775,7 @@ class CachingRepositoryData(RepositoryData):
         check.str_param(job_name, "job_name")
         return self._jobs.has_definition(job_name)
 
-    def get_all_pipelines(self):
+    def get_all_pipelines(self) -> List[PipelineDefinition]:
         """Return all pipelines/jobs in the repository as a list.
 
         Note that this will construct any pipeline/job that has not yet been constructed.
@@ -713,11 +787,15 @@ class CachingRepositoryData(RepositoryData):
             return self._all_pipelines
 
         self._all_jobs = self._jobs.get_all_definitions()
-        self._all_pipelines = self._pipelines.get_all_definitions() + self._all_jobs
-        self._check_solid_defs()
+        pipelines: List[PipelineDefinition] = [
+            *self._pipelines.get_all_definitions(),
+            *self._all_jobs,
+        ]
+        self._check_solid_defs(pipelines)
+        self._all_pipelines = pipelines
         return self._all_pipelines
 
-    def get_all_jobs(self):
+    def get_all_jobs(self) -> List[JobDefinition]:
         """Return all jobs in the repository as a list.
 
         Note that this will construct any job that has not yet been constructed.
@@ -733,9 +811,10 @@ class CachingRepositoryData(RepositoryData):
         # pipeline/job/graph uniqueness.
         self.get_all_pipelines()
 
-        return self._all_jobs
+        # The `get_all_pipelines` call ensures _all_jobs is set.
+        return cast(List[JobDefinition], self._all_jobs)
 
-    def get_pipeline(self, pipeline_name):
+    def get_pipeline(self, pipeline_name: str) -> PipelineDefinition:
         """Get a pipeline/job by name.
 
         If this pipeline/job has not yet been constructed, only this pipeline/job is constructed, and will
@@ -755,7 +834,7 @@ class CachingRepositoryData(RepositoryData):
         else:
             return self._pipelines.get_definition(pipeline_name)
 
-    def get_job(self, job_name):
+    def get_job(self, job_name: str) -> JobDefinition:
         """Get a job by name.
 
         If this job has not yet been constructed, only this job is constructed, and will
@@ -771,7 +850,7 @@ class CachingRepositoryData(RepositoryData):
         check.str_param(job_name, "job_name")
         return self._jobs.get_definition(job_name)
 
-    def get_partition_set_names(self):
+    def get_partition_set_names(self) -> List[str]:
         """Get the names of all partition sets in the repository.
 
         Returns:
@@ -779,7 +858,7 @@ class CachingRepositoryData(RepositoryData):
         """
         return self._partition_sets.get_definition_names()
 
-    def has_partition_set(self, partition_set_name):
+    def has_partition_set(self, partition_set_name: str) -> bool:
         """Check if a partition set with a given name is present in the repository.
 
         Args:
@@ -791,7 +870,7 @@ class CachingRepositoryData(RepositoryData):
         check.str_param(partition_set_name, "partition_set_name")
         return self._partition_sets.has_definition(partition_set_name)
 
-    def get_all_partition_sets(self):
+    def get_all_partition_sets(self) -> List[PartitionSetDefinition]:
         """Return all partition sets in the repository as a list.
 
         Note that this will construct any partition set that has not yet been constructed.
@@ -801,7 +880,7 @@ class CachingRepositoryData(RepositoryData):
         """
         return self._partition_sets.get_all_definitions()
 
-    def get_partition_set(self, partition_set_name):
+    def get_partition_set(self, partition_set_name: str) -> PartitionSetDefinition:
         """Get a partition set by name.
 
         If this partition set has not yet been constructed, only this partition set is constructed,
@@ -818,7 +897,7 @@ class CachingRepositoryData(RepositoryData):
 
         return self._partition_sets.get_definition(partition_set_name)
 
-    def get_schedule_names(self):
+    def get_schedule_names(self) -> List[str]:
         """Get the names of all schedules in the repository.
 
         Returns:
@@ -826,7 +905,7 @@ class CachingRepositoryData(RepositoryData):
         """
         return self._schedules.get_definition_names()
 
-    def get_all_schedules(self):
+    def get_all_schedules(self) -> List[ScheduleDefinition]:
         """Return all schedules in the repository as a list.
 
         Note that this will construct any schedule that has not yet been constructed.
@@ -836,7 +915,7 @@ class CachingRepositoryData(RepositoryData):
         """
         return self._schedules.get_all_definitions()
 
-    def get_schedule(self, schedule_name):
+    def get_schedule(self, schedule_name: str) -> ScheduleDefinition:
         """Get a schedule by name.
 
         if this schedule has not yet been constructed, only this schedule is constructed, and will
@@ -853,31 +932,31 @@ class CachingRepositoryData(RepositoryData):
 
         return self._schedules.get_definition(schedule_name)
 
-    def has_schedule(self, schedule_name):
+    def has_schedule(self, schedule_name: str) -> bool:
         check.str_param(schedule_name, "schedule_name")
 
         return self._schedules.has_definition(schedule_name)
 
-    def get_all_sensors(self):
+    def get_all_sensors(self) -> List[SensorDefinition]:
         return self._sensors.get_all_definitions()
 
-    def get_sensor_names(self):
+    def get_sensor_names(self) -> List[str]:
         return self._sensors.get_definition_names()
 
-    def get_sensor(self, sensor_name):
+    def get_sensor(self, sensor_name: str) -> SensorDefinition:
         return self._sensors.get_definition(sensor_name)
 
-    def has_sensor(self, sensor_name):
+    def has_sensor(self, sensor_name: str) -> bool:
         return self._sensors.has_definition(sensor_name)
 
-    def get_source_assets_by_key(self):
+    def get_source_assets_by_key(self) -> Mapping[AssetKey, SourceAsset]:
         return self._source_assets
 
-    def _check_solid_defs(self):
+    def _check_solid_defs(self, pipelines: List[PipelineDefinition]) -> None:
         solid_defs = {}
         solid_to_pipeline = {}
-        for pipeline in self._all_pipelines:
-            for solid_def in pipeline.all_node_defs + [pipeline.graph]:
+        for pipeline in pipelines:
+            for solid_def in [*pipeline.all_node_defs, pipeline.graph]:
                 # skip checks for subselected graphs because they don't have their own names
                 if isinstance(solid_def, SubselectedGraphDefinition):
                     break
@@ -899,13 +978,13 @@ class CachingRepositoryData(RepositoryData):
                         )
                     )
 
-    def _validate_pipeline(self, pipeline):
+    def _validate_pipeline(self, pipeline: PipelineDefinition) -> PipelineDefinition:
         return pipeline
 
-    def _validate_job(self, job):
+    def _validate_job(self, job: JobDefinition) -> JobDefinition:
         return job
 
-    def _validate_schedule(self, schedule):
+    def _validate_schedule(self, schedule: ScheduleDefinition) -> ScheduleDefinition:
         pipelines = self.get_pipeline_names()
 
         if schedule.pipeline_name not in pipelines:
@@ -916,7 +995,7 @@ class CachingRepositoryData(RepositoryData):
 
         return schedule
 
-    def _validate_sensor(self, sensor):
+    def _validate_sensor(self, sensor: SensorDefinition) -> SensorDefinition:
         pipelines = self.get_pipeline_names()
         if len(sensor.targets) == 0:
             # skip validation when the sensor does not target a pipeline
@@ -931,7 +1010,9 @@ class CachingRepositoryData(RepositoryData):
 
         return sensor
 
-    def _validate_partition_set(self, partition_set):
+    def _validate_partition_set(
+        self, partition_set: PartitionSetDefinition
+    ) -> PartitionSetDefinition:
         return partition_set
 
 
@@ -958,24 +1039,24 @@ class RepositoryDefinition:
         self._repository_data = check.inst_param(repository_data, "repository_data", RepositoryData)
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self._name
 
     @property
-    def description(self):
+    def description(self) -> Optional[str]:
         return self._description
 
     @property
-    def pipeline_names(self):
+    def pipeline_names(self) -> List[str]:
         """List[str]: Names of all pipelines/jobs in the repository"""
         return self._repository_data.get_pipeline_names()
 
     @property
-    def job_names(self):
+    def job_names(self) -> List[str]:
         """List[str]: Names of all jobs in the repository"""
         return self._repository_data.get_job_names()
 
-    def has_pipeline(self, name):
+    def has_pipeline(self, name: str) -> bool:
         """Check if a pipeline/job with a given name is present in the repository.
 
         Args:
@@ -986,7 +1067,7 @@ class RepositoryDefinition:
         """
         return self._repository_data.has_pipeline(name)
 
-    def get_pipeline(self, name):
+    def get_pipeline(self, name: str) -> PipelineDefinition:
         """Get a pipeline/job by name.
 
         If this pipeline/job is present in the lazily evaluated dictionary passed to the
@@ -1001,7 +1082,7 @@ class RepositoryDefinition:
         """
         return self._repository_data.get_pipeline(name)
 
-    def get_all_pipelines(self):
+    def get_all_pipelines(self) -> List[PipelineDefinition]:
         """Return all pipelines/jobs in the repository as a list.
 
         Note that this will construct any pipeline/job in the lazily evaluated dictionary that
@@ -1012,7 +1093,7 @@ class RepositoryDefinition:
         """
         return self._repository_data.get_all_pipelines()
 
-    def has_job(self, name):
+    def has_job(self, name: str) -> bool:
         """Check if a job with a given name is present in the repository.
 
         Args:
@@ -1023,7 +1104,7 @@ class RepositoryDefinition:
         """
         return self._repository_data.has_job(name)
 
-    def get_job(self, name):
+    def get_job(self, name: str) -> JobDefinition:
         """Get a job by name.
 
         If this job is present in the lazily evaluated dictionary passed to the
@@ -1039,7 +1120,7 @@ class RepositoryDefinition:
         """
         return self._repository_data.get_job(name)
 
-    def get_all_jobs(self):
+    def get_all_jobs(self) -> List[JobDefinition]:
         """Return all jobs in the repository as a list.
 
         Note that this will construct any job in the lazily evaluated dictionary that has
@@ -1051,34 +1132,34 @@ class RepositoryDefinition:
         return self._repository_data.get_all_jobs()
 
     @property
-    def partition_set_defs(self):
+    def partition_set_defs(self) -> List[PartitionSetDefinition]:
         return self._repository_data.get_all_partition_sets()
 
-    def get_partition_set_def(self, name):
+    def get_partition_set_def(self, name: str) -> PartitionSetDefinition:
         return self._repository_data.get_partition_set(name)
 
     @property
-    def schedule_defs(self):
+    def schedule_defs(self) -> List[ScheduleDefinition]:
         return self._repository_data.get_all_schedules()
 
-    def get_schedule_def(self, name):
+    def get_schedule_def(self, name: str) -> ScheduleDefinition:
         return self._repository_data.get_schedule(name)
 
-    def has_schedule_def(self, name):
+    def has_schedule_def(self, name: str) -> bool:
         return self._repository_data.has_schedule(name)
 
     @property
-    def sensor_defs(self):
+    def sensor_defs(self) -> List[SensorDefinition]:
         return self._repository_data.get_all_sensors()
 
-    def get_sensor_def(self, name):
+    def get_sensor_def(self, name: str) -> SensorDefinition:
         return self._repository_data.get_sensor(name)
 
-    def has_sensor_def(self, name):
+    def has_sensor_def(self, name: str) -> bool:
         return self._repository_data.has_sensor(name)
 
     @property
-    def source_assets_by_key(self):
+    def source_assets_by_key(self) -> Dict[AssetKey, SourceAsset]:
         return self._repository_data.get_source_assets_by_key()
 
     # If definition comes from the @repository decorator, then the __call__ method will be
