@@ -9,13 +9,14 @@ from dagster.core.definitions import (
     AssetKey,
     AssetMaterialization,
     AssetObservation,
-    EventMetadataEntry,
     ExpectationResult,
     HookDefinition,
     Materialization,
+    MetadataEntry,
     NodeHandle,
 )
 from dagster.core.definitions.events import AssetLineageInfo, ObjectStoreOperationType
+from dagster.core.definitions.metadata import MetadataValue
 from dagster.core.errors import DagsterError, HookExecutionError
 from dagster.core.execution.context.hook import HookContext
 from dagster.core.execution.context.system import (
@@ -34,15 +35,11 @@ from dagster.utils.error import SerializableErrorInfo, serializable_error_info_f
 from dagster.utils.timing import format_duration
 
 if TYPE_CHECKING:
+    from dagster.core.definitions.events import ObjectStoreOperation
+    from dagster.core.execution.plan.inputs import StepInputData
+    from dagster.core.execution.plan.objects import StepFailureData, StepRetryData, StepSuccessData
     from dagster.core.execution.plan.plan import ExecutionPlan
     from dagster.core.execution.plan.step import ExecutionStep, StepKind
-    from dagster.core.execution.plan.inputs import StepInputData
-    from dagster.core.execution.plan.objects import (
-        StepSuccessData,
-        StepFailureData,
-        StepRetryData,
-    )
-    from dagster.core.definitions.events import ObjectStoreOperation
 
     EventSpecificData = Union[
         StepOutputData,
@@ -118,6 +115,8 @@ class DagsterEventType(Enum):
 
     ALERT_START = "ALERT_START"
     ALERT_SUCCESS = "ALERT_SUCCESS"
+    ALERT_FAILURE = "ALERT_FAILURE"
+
     LOGS_CAPTURED = "LOGS_CAPTURED"
 
 
@@ -175,6 +174,7 @@ HOOK_EVENTS = {
 ALERT_EVENTS = {
     DagsterEventType.ALERT_START,
     DagsterEventType.ALERT_SUCCESS,
+    DagsterEventType.ALERT_FAILURE,
 }
 
 
@@ -190,6 +190,11 @@ EVENT_TYPE_TO_PIPELINE_RUN_STATUS = {
 
 PIPELINE_RUN_STATUS_TO_EVENT_TYPE = {v: k for k, v in EVENT_TYPE_TO_PIPELINE_RUN_STATUS.items()}
 
+ASSET_EVENTS = {
+    DagsterEventType.ASSET_MATERIALIZATION,
+    DagsterEventType.ASSET_OBSERVATION,
+}
+
 
 def _assert_type(
     method: str, expected_type: DagsterEventType, actual_type: DagsterEventType
@@ -205,8 +210,8 @@ def _assert_type(
 def _validate_event_specific_data(
     event_type: DagsterEventType, event_specific_data: Optional["EventSpecificData"]
 ) -> Optional["EventSpecificData"]:
-    from dagster.core.execution.plan.objects import StepFailureData, StepSuccessData
     from dagster.core.execution.plan.inputs import StepInputData
+    from dagster.core.execution.plan.objects import StepFailureData, StepSuccessData
 
     if event_type == DagsterEventType.STEP_OUTPUT:
         check.inst_param(event_specific_data, "event_specific_data", StepOutputData)
@@ -494,6 +499,10 @@ class DagsterEvent(
     def is_step_materialization(self) -> bool:
         return self.event_type == DagsterEventType.ASSET_MATERIALIZATION
 
+    @property
+    def is_expectation_result(self) -> bool:
+        return self.event_type == DagsterEventType.STEP_EXPECTATION_RESULT
+
     @property
     def is_asset_observation(self) -> bool:
         return self.event_type == DagsterEventType.ASSET_OBSERVATION
@@ -728,7 +737,7 @@ class DagsterEvent(
     def asset_materialization(
         step_context: IStepContext,
         materialization: Union[AssetMaterialization, Materialization],
-        asset_lineage: List[AssetLineageInfo] = None,
+        asset_lineage: Optional[List[AssetLineageInfo]] = None,
     ) -> "DagsterEvent":
         return DagsterEvent.from_step(
             event_type=DagsterEventType.ASSET_MATERIALIZATION,
@@ -869,13 +878,15 @@ class DagsterEvent(
     ) -> "DagsterEvent":
 
         metadata_entries = []
-        for resource_key in resource_instances.keys():
-            resource_obj = resource_instances[resource_key]
-            resource_time = resource_init_times[resource_key]
-            metadata_entries.append(
-                EventMetadataEntry.python_artifact(
-                    resource_obj.__class__, resource_key, "Initialized in {}".format(resource_time)
-                )
+        for key in resource_instances.keys():
+            metadata_entries.extend(
+                [
+                    MetadataEntry(
+                        key,
+                        value=MetadataValue.python_artifact(resource_instances[key].__class__),
+                    ),
+                    MetadataEntry(f"{key}:init_time", value=resource_init_times[key]),
+                ]
             )
 
         return DagsterEvent.from_resource(
@@ -939,7 +950,7 @@ class DagsterEvent(
         pipeline_context: IPlanContext,
         message: str,
         event_specific_data: Optional["EngineEventData"] = None,
-        step_handle: Optional[StepHandle] = None,
+        step_handle: Optional[Union[StepHandle, ResolvedFromDynamicStepHandle]] = None,
     ) -> "DagsterEvent":
         return DagsterEvent.from_pipeline(
             DagsterEventType.ENGINE_EVENT,
@@ -1018,7 +1029,9 @@ class DagsterEvent(
                 value_name=value_name,
                 address=object_store_operation_result.key,
                 metadata_entries=[
-                    EventMetadataEntry.path(object_store_operation_result.key, label="key")
+                    MetadataEntry(
+                        "key", value=MetadataValue.path(object_store_operation_result.key)
+                    ),
                 ],
                 version=object_store_operation_result.version,
                 mapping_key=object_store_operation_result.mapping_key,
@@ -1032,7 +1045,7 @@ class DagsterEvent(
         output_name: str,
         manager_key: str,
         message_override: Optional[str] = None,
-        metadata_entries: Optional[List[EventMetadataEntry]] = None,
+        metadata_entries: Optional[List[MetadataEntry]] = None,
     ) -> "DagsterEvent":
         message = f'Handled output "{output_name}" using IO manager "{manager_key}"'
         return DagsterEvent.from_step(
@@ -1256,7 +1269,7 @@ class ObjectStoreOperationResultData(
         [
             ("op", ObjectStoreOperationType),
             ("value_name", Optional[str]),
-            ("metadata_entries", List[EventMetadataEntry]),
+            ("metadata_entries", List[MetadataEntry]),
             ("address", Optional[str]),
             ("version", Optional[str]),
             ("mapping_key", Optional[str]),
@@ -1267,7 +1280,7 @@ class ObjectStoreOperationResultData(
         cls,
         op: ObjectStoreOperationType,
         value_name: Optional[str] = None,
-        metadata_entries: Optional[List[EventMetadataEntry]] = None,
+        metadata_entries: Optional[List[MetadataEntry]] = None,
         address: Optional[str] = None,
         version: Optional[str] = None,
         mapping_key: Optional[str] = None,
@@ -1277,7 +1290,7 @@ class ObjectStoreOperationResultData(
             op=cast(ObjectStoreOperationType, check.str_param(op, "op")),
             value_name=check.opt_str_param(value_name, "value_name"),
             metadata_entries=check.opt_list_param(
-                metadata_entries, "metadata_entries", of_type=EventMetadataEntry
+                metadata_entries, "metadata_entries", of_type=MetadataEntry
             ),
             address=check.opt_str_param(address, "address"),
             version=check.opt_str_param(version, "version"),
@@ -1290,7 +1303,7 @@ class EngineEventData(
     NamedTuple(
         "_EngineEventData",
         [
-            ("metadata_entries", List[EventMetadataEntry]),
+            ("metadata_entries", List[MetadataEntry]),
             ("error", Optional[SerializableErrorInfo]),
             ("marker_start", Optional[str]),
             ("marker_end", Optional[str]),
@@ -1303,7 +1316,7 @@ class EngineEventData(
     #
     def __new__(
         cls,
-        metadata_entries: Optional[List[EventMetadataEntry]] = None,
+        metadata_entries: Optional[List[MetadataEntry]] = None,
         error: Optional[SerializableErrorInfo] = None,
         marker_start: Optional[str] = None,
         marker_end: Optional[str] = None,
@@ -1311,7 +1324,7 @@ class EngineEventData(
         return super(EngineEventData, cls).__new__(
             cls,
             metadata_entries=check.opt_list_param(
-                metadata_entries, "metadata_entries", of_type=EventMetadataEntry
+                metadata_entries, "metadata_entries", of_type=MetadataEntry
             ),
             error=check.opt_inst_param(error, "error", SerializableErrorInfo),
             marker_start=check.opt_str_param(marker_start, "marker_start"),
@@ -1323,9 +1336,9 @@ class EngineEventData(
         pid: int, step_keys_to_execute: Optional[List[str]] = None, marker_end: Optional[str] = None
     ) -> "EngineEventData":
         return EngineEventData(
-            metadata_entries=[EventMetadataEntry.text(str(pid), "pid")]
+            metadata_entries=[MetadataEntry("pid", value=str(pid))]
             + (
-                [EventMetadataEntry.text(str(step_keys_to_execute), "step_keys")]
+                [MetadataEntry("step_keys", value=str(step_keys_to_execute))]
                 if step_keys_to_execute
                 else []
             ),
@@ -1337,9 +1350,9 @@ class EngineEventData(
         pid: int, step_keys_to_execute: Optional[List[str]] = None
     ) -> "EngineEventData":
         return EngineEventData(
-            metadata_entries=[EventMetadataEntry.text(str(pid), "pid")]
+            metadata_entries=[MetadataEntry("pid", value=str(pid))]
             + (
-                [EventMetadataEntry.text(str(step_keys_to_execute), "step_keys")]
+                [MetadataEntry("step_keys", value=str(step_keys_to_execute))]
                 if step_keys_to_execute
                 else []
             )
@@ -1348,7 +1361,7 @@ class EngineEventData(
     @staticmethod
     def interrupted(steps_interrupted: List[str]) -> "EngineEventData":
         return EngineEventData(
-            metadata_entries=[EventMetadataEntry.text(str(steps_interrupted), "steps_interrupted")]
+            metadata_entries=[MetadataEntry("steps_interrupted", value=str(steps_interrupted))]
         )
 
     @staticmethod
@@ -1408,7 +1421,7 @@ class HandledOutputData(
         [
             ("output_name", str),
             ("manager_key", str),
-            ("metadata_entries", List[EventMetadataEntry]),
+            ("metadata_entries", List[MetadataEntry]),
         ],
     )
 ):
@@ -1416,14 +1429,14 @@ class HandledOutputData(
         cls,
         output_name: str,
         manager_key: str,
-        metadata_entries: Optional[List[EventMetadataEntry]] = None,
+        metadata_entries: Optional[List[MetadataEntry]] = None,
     ):
         return super(HandledOutputData, cls).__new__(
             cls,
             output_name=check.str_param(output_name, "output_name"),
             manager_key=check.str_param(manager_key, "manager_key"),
             metadata_entries=check.opt_list_param(
-                metadata_entries, "metadata_entries", of_type=EventMetadataEntry
+                metadata_entries, "metadata_entries", of_type=MetadataEntry
             ),
         )
 
