@@ -1,5 +1,5 @@
 import hashlib
-from abc import ABC, abstractmethod, abstractproperty
+from abc import ABC, abstractmethod
 from typing import TYPE_CHECKING, Any, Dict, Iterator, List, NamedTuple, Optional, Set, Union, cast
 
 from dagster import check
@@ -21,11 +21,11 @@ from .outputs import StepOutputHandle, UnresolvedStepOutputHandle
 from .utils import build_resources_for_manager, solid_execution_error_boundary
 
 if TYPE_CHECKING:
-    from dagster.core.types.dagster_type import DagsterType
-    from dagster.core.storage.input_manager import InputManager
     from dagster.core.events import DagsterEvent
-    from dagster.core.execution.context.system import StepExecutionContext
     from dagster.core.execution.context.input import InputContext
+    from dagster.core.execution.context.system import StepExecutionContext
+    from dagster.core.storage.input_manager import InputManager
+    from dagster.core.types.dagster_type import DagsterType
 
 
 def _get_asset_lineage_from_fns(
@@ -102,11 +102,13 @@ class StepInputSource(ABC):
     def get_input_def(self, pipeline_def: PipelineDefinition) -> InputDefinition:
         return pipeline_def.get_solid(self.solid_handle).input_def_named(self.input_name)
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def solid_handle(self) -> NodeHandle:
         pass
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def input_name(self) -> str:
         pass
 
@@ -166,7 +168,7 @@ class FromRootInputManager(
         )
 
     def compute_version(self, step_versions, pipeline_def, resolved_run_config) -> Optional[str]:
-        from ..resolve_versions import resolve_config_version, check_valid_version
+        from ..resolve_versions import check_valid_version, resolve_config_version
 
         solid = pipeline_def.get_solid(self.solid_handle)
         root_manager_key = solid.input_def_named(self.input_name).root_manager_key
@@ -262,7 +264,6 @@ class FromStepOutput(
 
     def load_input_object(self, step_context: "StepExecutionContext") -> Iterator["DagsterEvent"]:
         from dagster.core.events import DagsterEvent
-        from dagster.core.storage.intermediate_storage import IntermediateStorageAdapter
 
         source_handle = self.step_output_handle
         manager_key = step_context.execution_plan.get_manager_key(
@@ -284,9 +285,6 @@ class FromStepOutput(
             manager_key=manager_key,
             upstream_output_name=source_handle.output_name,
             upstream_step_key=source_handle.step_key,
-            message_override=f'Loaded input "{self.input_name}" using intermediate storage'
-            if isinstance(input_manager, IntermediateStorageAdapter)
-            else None,
         )
 
     def compute_version(
@@ -367,6 +365,7 @@ class FromConfig(
                 f'Error occurred while loading input "{self.input_name}" of '
                 f'step "{step_context.step.key}":'
             ),
+            log_manager=step_context.log,
         ):
             dagster_type = self.get_input_def(step_context.pipeline_def).dagster_type
 
@@ -397,6 +396,62 @@ class FromConfig(
         return dagster_type.loader.compute_loaded_input_version(config_data)
 
 
+@whitelist_for_serdes
+class FromRootInputConfig(
+    NamedTuple("_FromRootInputConfig", [("input_name", str)]),
+    StepInputSource,
+):
+    """This step input source is configuration to be passed to a type loader"""
+
+    def __new__(cls, input_name: str):
+        return super(FromRootInputConfig, cls).__new__(
+            cls,
+            input_name=input_name,
+        )
+
+    def get_input_def(self, pipeline_def: PipelineDefinition) -> InputDefinition:
+        return pipeline_def.graph.input_def_named(self.input_name)
+
+    def load_input_object(self, step_context: "StepExecutionContext") -> Any:
+        with user_code_error_boundary(
+            DagsterTypeLoadingError,
+            msg_fn=lambda: (f'Error occurred while loading top-level input "{self.input_name}": '),
+            log_manager=step_context.log,
+        ):
+            dagster_type = self.get_input_def(step_context.pipeline_def).dagster_type
+
+            input_config = step_context.resolved_run_config.inputs
+            config_data = input_config.get(self.input_name) if input_config else None
+
+            return dagster_type.loader.construct_from_config_value(step_context, config_data)
+
+    def required_resource_keys(self, pipeline_def: PipelineDefinition) -> Set[str]:
+        input_def = self.get_input_def(pipeline_def)
+        return (
+            input_def.dagster_type.loader.required_resource_keys()
+            if input_def.dagster_type.loader
+            else set()
+        )
+
+    @property
+    def solid_handle(self):
+        raise DagsterInvariantViolationError(
+            "Solid handle is not set on the root input config source."
+        )
+
+    def compute_version(
+        self,
+        step_versions: Dict[str, Optional[str]],
+        pipeline_def: PipelineDefinition,
+        resolved_run_config: ResolvedRunConfig,
+    ) -> Optional[str]:
+        input_config = resolved_run_config.inputs
+        config_data = input_config.get(self.input_name) if input_config else None
+
+        dagster_type = self.get_input_def(pipeline_def).dagster_type
+        return dagster_type.loader.compute_loaded_input_version(config_data)
+
+
 @whitelist_for_serdes
 class FromDefaultValue(
     NamedTuple(
@@ -472,6 +527,7 @@ class FromMultipleSources(
         from dagster.core.events import DagsterEvent
 
         values = []
+
         # some upstream steps may have skipped and we allow fan-in to continue in their absence
         source_handles_to_skip = list(
             filter(lambda x: not step_context.can_load(x), self.step_output_handle_dependencies)
@@ -751,9 +807,11 @@ class UnresolvedCollectStepInput(NamedTuple):
         return [self.source.get_step_output_handle_dep_with_placeholder()]
 
 
-StepInputSourceTypes = (
+StepInputSourceUnion = Union[
     StepInputSource,
     FromDynamicCollect,
     FromUnresolvedStepOutput,
     FromPendingDynamicStepOutput,
-)
+]
+
+StepInputSourceTypes = StepInputSourceUnion.__args__  # type: ignore
