@@ -1,13 +1,13 @@
 import os
 import sys
 from abc import ABC, abstractmethod
-from collections import namedtuple
 from contextlib import contextmanager
 from inspect import Parameter
 from typing import (
     TYPE_CHECKING,
     Any,
     Dict,
+    Generator,
     Mapping,
     NamedTuple,
     NoReturn,
@@ -18,7 +18,7 @@ from typing import (
 )
 
 from dagster import check
-from dagster.core.definitions.reconstructable import ReconstructableRepository
+from dagster.core.definitions.reconstruct import ReconstructableRepository
 from dagster.core.errors import DagsterInvariantViolationError, DagsterUserCodeUnreachableError
 from dagster.core.types.loadable_target_origin import LoadableTargetOrigin
 from dagster.serdes import (
@@ -32,7 +32,10 @@ from dagster.serdes.serdes import WhitelistMap, unpack_inner_value
 if TYPE_CHECKING:
     from dagster.core.host_representation.repository_location import (
         GrpcServerRepositoryLocation,
+        InProcessRepositoryLocation,
+        RepositoryLocation,
     )
+    from dagster.grpc.client import DagsterGrpcClient
 
 # This is a hard-coded name for the special "in-process" location.
 # This is typically only used for test, although we may allow
@@ -76,27 +79,27 @@ def _assign_loadable_target_origin_name(loadable_target_origin: LoadableTargetOr
     )
 
 
-class RepositoryLocationOrigin(ABC):
+class RepositoryLocationOrigin(ABC, tuple):
     """Serializable representation of a RepositoryLocation that can be used to
     uniquely identify the location or reload it in across process boundaries.
     """
 
     @property
-    def is_reload_supported(self):
+    def is_reload_supported(self) -> bool:
         return True
 
     @property
-    def is_shutdown_supported(self):
+    def is_shutdown_supported(self) -> bool:
         return False
 
-    def shutdown_server(self):
+    def shutdown_server(self) -> None:
         raise NotImplementedError
 
     @abstractmethod
-    def get_display_metadata(self):
+    def get_display_metadata(self) -> Dict[str, Any]:
         pass
 
-    def get_id(self):
+    def get_id(self) -> str:
         return create_snapshot_id(self)
 
     @property
@@ -105,7 +108,7 @@ class RepositoryLocationOrigin(ABC):
         pass
 
     @abstractmethod
-    def create_location(self):
+    def create_location(self) -> "RepositoryLocation":
         pass
 
     @property
@@ -123,13 +126,13 @@ class RegisteredRepositoryLocationOrigin(
     its own mapping from location name to repository location metadata.
     """
 
-    def __new__(cls, location_name):
+    def __new__(cls, location_name: str):
         return super(RegisteredRepositoryLocationOrigin, cls).__new__(cls, location_name)
 
-    def get_display_metadata(self):
+    def get_display_metadata(self) -> Dict[str, Any]:
         return {}
 
-    def create_location(self):
+    def create_location(self) -> NoReturn:
         raise DagsterInvariantViolationError(
             "A RegisteredRepositoryLocationOrigin does not have enough information to load its "
             "repository location on its own."
@@ -145,7 +148,7 @@ class InProcessRepositoryLocationOrigin(
     used in tests.
     """
 
-    def __new__(cls, recon_repo):
+    def __new__(cls, recon_repo: ReconstructableRepository):
         return super(InProcessRepositoryLocationOrigin, cls).__new__(
             cls, check.inst_param(recon_repo, "recon_repo", ReconstructableRepository)
         )
@@ -155,15 +158,15 @@ class InProcessRepositoryLocationOrigin(
         return IN_PROCESS_NAME
 
     @property
-    def is_reload_supported(self):
+    def is_reload_supported(self) -> bool:
         return False
 
-    def get_display_metadata(self):
+    def get_display_metadata(self) -> Dict[str, Any]:
         return {
             "in_process_code_pointer": self.recon_repo.pointer.describe(),
         }
 
-    def create_location(self):
+    def create_location(self) -> "InProcessRepositoryLocation":
         from dagster.core.host_representation.repository_location import InProcessRepositoryLocation
 
         return InProcessRepositoryLocation(self)
@@ -181,7 +184,9 @@ class ManagedGrpcPythonEnvRepositoryLocationOrigin(
     for these repository locations on startup.
     """
 
-    def __new__(cls, loadable_target_origin, location_name=None):
+    def __new__(
+        cls, loadable_target_origin: LoadableTargetOrigin, location_name: Optional[str] = None
+    ):
         return super(ManagedGrpcPythonEnvRepositoryLocationOrigin, cls).__new__(
             cls,
             check.inst_param(
@@ -214,14 +219,15 @@ class ManagedGrpcPythonEnvRepositoryLocationOrigin(
         )
 
     @contextmanager
-    def create_single_location(self):
-        from .repository_location import GrpcServerRepositoryLocation
-        from .grpc_server_registry import ProcessGrpcServerRegistry
+    def create_single_location(self) -> Generator["RepositoryLocation", None, None]:
         from dagster.core.workspace.context import (
             DAGIT_GRPC_SERVER_HEARTBEAT_TTL,
             DAGIT_GRPC_SERVER_STARTUP_TIMEOUT,
         )
 
+        from .grpc_server_registry import ProcessGrpcServerRegistry
+        from .repository_location import GrpcServerRepositoryLocation
+
         with ProcessGrpcServerRegistry(
             reload_interval=0,
             heartbeat_ttl=DAGIT_GRPC_SERVER_HEARTBEAT_TTL,
@@ -265,7 +271,14 @@ class GrpcServerRepositoryLocationOrigin(
     is not responsible for managing the lifecycle of the server.
     """
 
-    def __new__(cls, host, port=None, socket=None, location_name=None, use_ssl=None):
+    def __new__(
+        cls,
+        host: str,
+        port: Optional[int] = None,
+        socket: Optional[str] = None,
+        location_name: Optional[str] = None,
+        use_ssl: Optional[bool] = None,
+    ):
         return super(GrpcServerRepositoryLocationOrigin, cls).__new__(
             cls,
             check.str_param(host, "host"),
@@ -293,10 +306,10 @@ class GrpcServerRepositoryLocationOrigin(
         return GrpcServerRepositoryLocation(self)
 
     @property
-    def supports_server_watch(self):
+    def supports_server_watch(self) -> bool:
         return True
 
-    def create_client(self):
+    def create_client(self) -> "DagsterGrpcClient":
         from dagster.grpc.client import DagsterGrpcClient
 
         return DagsterGrpcClient(
@@ -307,10 +320,10 @@ class GrpcServerRepositoryLocationOrigin(
         )
 
     @property
-    def is_shutdown_supported(self):
+    def is_shutdown_supported(self) -> bool:
         return True
 
-    def shutdown_server(self):
+    def shutdown_server(self) -> None:
         try:
             self.create_client().shutdown_server()
         except DagsterUserCodeUnreachableError:
@@ -320,13 +333,16 @@ class GrpcServerRepositoryLocationOrigin(
 
 @whitelist_for_serdes
 class ExternalRepositoryOrigin(
-    namedtuple("_ExternalRepositoryOrigin", "repository_location_origin repository_name")
+    NamedTuple(
+        "_ExternalRepositoryOrigin",
+        [("repository_location_origin", RepositoryLocationOrigin), ("repository_name", str)],
+    )
 ):
     """Serializable representation of an ExternalRepository that can be used to
     uniquely it or reload it in across process boundaries.
     """
 
-    def __new__(cls, repository_location_origin, repository_name):
+    def __new__(cls, repository_location_origin: RepositoryLocationOrigin, repository_name: str):
         return super(ExternalRepositoryOrigin, cls).__new__(
             cls,
             check.inst_param(
@@ -335,28 +351,31 @@ class ExternalRepositoryOrigin(
             check.str_param(repository_name, "repository_name"),
         )
 
-    def get_id(self):
+    def get_id(self) -> str:
         return create_snapshot_id(self)
 
-    def get_pipeline_origin(self, pipeline_name):
+    def get_pipeline_origin(self, pipeline_name: str) -> "ExternalPipelineOrigin":
         return ExternalPipelineOrigin(self, pipeline_name)
 
-    def get_instigator_origin(self, instigator_name):
+    def get_instigator_origin(self, instigator_name: str) -> "ExternalInstigatorOrigin":
         return ExternalInstigatorOrigin(self, instigator_name)
 
-    def get_partition_set_origin(self, partition_set_name):
+    def get_partition_set_origin(self, partition_set_name: str) -> "ExternalPartitionSetOrigin":
         return ExternalPartitionSetOrigin(self, partition_set_name)
 
 
 @whitelist_for_serdes
 class ExternalPipelineOrigin(
-    namedtuple("_ExternalPipelineOrigin", "external_repository_origin pipeline_name")
+    NamedTuple(
+        "_ExternalPipelineOrigin",
+        [("external_repository_origin", ExternalRepositoryOrigin), ("pipeline_name", str)],
+    )
 ):
     """Serializable representation of an ExternalPipeline that can be used to
     uniquely it or reload it in across process boundaries.
     """
 
-    def __new__(cls, external_repository_origin, pipeline_name):
+    def __new__(cls, external_repository_origin: ExternalRepositoryOrigin, pipeline_name: str):
         return super(ExternalPipelineOrigin, cls).__new__(
             cls,
             check.inst_param(
@@ -367,7 +386,7 @@ class ExternalPipelineOrigin(
             check.str_param(pipeline_name, "pipeline_name"),
         )
 
-    def get_id(self):
+    def get_id(self) -> str:
         return create_snapshot_id(self)
 
 
@@ -420,13 +439,16 @@ class ExternalInstigatorOriginSerializer(DefaultNamedTupleSerializer):
 
 @whitelist_for_serdes(serializer=ExternalInstigatorOriginSerializer)
 class ExternalInstigatorOrigin(
-    namedtuple("_ExternalInstigatorOrigin", "external_repository_origin instigator_name")
+    NamedTuple(
+        "_ExternalInstigatorOrigin",
+        [("external_repository_origin", ExternalRepositoryOrigin), ("instigator_name", str)],
+    )
 ):
     """Serializable representation of an ExternalJob that can be used to
     uniquely it or reload it in across process boundaries.
     """
 
-    def __new__(cls, external_repository_origin, instigator_name):
+    def __new__(cls, external_repository_origin: ExternalRepositoryOrigin, instigator_name: str):
         return super(ExternalInstigatorOrigin, cls).__new__(
             cls,
             check.inst_param(
@@ -437,7 +459,7 @@ class ExternalInstigatorOrigin(
             check.str_param(instigator_name, "instigator_name"),
         )
 
-    def get_id(self):
+    def get_id(self) -> str:
         return create_snapshot_id(self)
 
 
@@ -451,13 +473,16 @@ register_serdes_tuple_fallbacks({"ExternalJobOrigin": ExternalInstigatorOrigin})
 
 @whitelist_for_serdes
 class ExternalPartitionSetOrigin(
-    namedtuple("_PartitionSetOrigin", "external_repository_origin partition_set_name")
+    NamedTuple(
+        "_PartitionSetOrigin",
+        [("external_repository_origin", ExternalRepositoryOrigin), ("partition_set_name", str)],
+    )
 ):
     """Serializable representation of an ExternalPartitionSet that can be used to
     uniquely it or reload it in across process boundaries.
     """
 
-    def __new__(cls, external_repository_origin, partition_set_name):
+    def __new__(cls, external_repository_origin: ExternalRepositoryOrigin, partition_set_name: str):
         return super(ExternalPartitionSetOrigin, cls).__new__(
             cls,
             check.inst_param(
@@ -468,5 +493,5 @@ class ExternalPartitionSetOrigin(
             check.str_param(partition_set_name, "partition_set_name"),
         )
 
-    def get_id(self):
+    def get_id(self) -> str:
         return create_snapshot_id(self)
