@@ -1,17 +1,29 @@
-from collections import namedtuple
+from typing import List, NamedTuple, Optional
 
 from dagster import check
 
 
 class PipelineSelector(
-    namedtuple("_PipelineSelector", "location_name repository_name pipeline_name solid_selection")
+    NamedTuple(
+        "_PipelineSelector",
+        [
+            ("location_name", str),
+            ("repository_name", str),
+            ("pipeline_name", str),
+            ("solid_selection", Optional[List[str]]),
+        ],
+    )
 ):
     """
     The information needed to resolve a pipeline within a host process.
     """
 
     def __new__(
-        cls, location_name, repository_name, pipeline_name, solid_selection,
+        cls,
+        location_name: str,
+        repository_name: str,
+        pipeline_name: str,
+        solid_selection: Optional[List[str]],
     ):
         return super(PipelineSelector, cls).__new__(
             cls,
@@ -41,8 +53,10 @@ class PipelineSelector(
         )
 
 
-class RepositorySelector(namedtuple("_RepositorySelector", "location_name repository_name")):
-    def __new__(cls, location_name, repository_name):
+class RepositorySelector(
+    NamedTuple("_RepositorySelector", [("location_name", str), ("repository_name", str)])
+):
+    def __new__(cls, location_name: str, repository_name: str):
         return super(RepositorySelector, cls).__new__(
             cls,
             location_name=check.str_param(location_name, "location_name"),
@@ -64,9 +78,12 @@ class RepositorySelector(namedtuple("_RepositorySelector", "location_name reposi
 
 
 class ScheduleSelector(
-    namedtuple("_ScheduleSelector", "location_name repository_name schedule_name")
+    NamedTuple(
+        "_ScheduleSelector",
+        [("location_name", str), ("repository_name", str), ("schedule_name", str)],
+    )
 ):
-    def __new__(cls, location_name, repository_name, schedule_name):
+    def __new__(cls, location_name: str, repository_name: str, schedule_name: str):
         return super(ScheduleSelector, cls).__new__(
             cls,
             location_name=check.str_param(location_name, "location_name"),
@@ -90,28 +107,84 @@ class ScheduleSelector(
         )
 
 
-class TriggerSelector(
-    namedtuple("_TriggerSelector", "location_name repository_name executable_name")
+class SensorSelector(
+    NamedTuple(
+        "_SensorSelector", [("location_name", str), ("repository_name", str), ("sensor_name", str)]
+    )
 ):
-    def __new__(cls, location_name, repository_name, executable_name):
-        return super(TriggerSelector, cls).__new__(
+    def __new__(cls, location_name: str, repository_name: str, sensor_name: str):
+        return super(SensorSelector, cls).__new__(
             cls,
             location_name=check.str_param(location_name, "location_name"),
             repository_name=check.str_param(repository_name, "repository_name"),
-            executable_name=check.str_param(executable_name, "executable_name"),
+            sensor_name=check.str_param(sensor_name, "sensor_name"),
         )
 
     def to_graphql_input(self):
         return {
             "repositoryLocationName": self.location_name,
             "repositoryName": self.repository_name,
-            "executableName": self.executable_name,
+            "sensorName": self.sensor_name,
         }
 
     @staticmethod
     def from_graphql_input(graphql_data):
-        return TriggerSelector(
+        return SensorSelector(
             location_name=graphql_data["repositoryLocationName"],
             repository_name=graphql_data["repositoryName"],
-            executable_name=graphql_data["executableName"],
+            sensor_name=graphql_data["sensorName"],
         )
+
+
+class InstigationSelector(
+    NamedTuple(
+        "_InstigationSelector", [("location_name", str), ("repository_name", str), ("name", str)]
+    )
+):
+    def __new__(cls, location_name: str, repository_name: str, name: str):
+        return super(InstigationSelector, cls).__new__(
+            cls,
+            location_name=check.str_param(location_name, "location_name"),
+            repository_name=check.str_param(repository_name, "repository_name"),
+            name=check.str_param(name, "name"),
+        )
+
+    def to_graphql_input(self):
+        return {
+            "repositoryLocationName": self.location_name,
+            "repositoryName": self.repository_name,
+            "name": self.name,
+        }
+
+    @staticmethod
+    def from_graphql_input(graphql_data):
+        return InstigationSelector(
+            location_name=graphql_data["repositoryLocationName"],
+            repository_name=graphql_data["repositoryName"],
+            name=graphql_data["name"],
+        )
+
+
+class GraphSelector(
+    NamedTuple(
+        "_GraphSelector", [("location_name", str), ("repository_name", str), ("graph_name", str)]
+    )
+):
+    """
+    The information needed to resolve a graph within a host process.
+    """
+
+    def __new__(cls, location_name: str, repository_name: str, graph_name: str):
+        return super(GraphSelector, cls).__new__(
+            cls,
+            location_name=check.str_param(location_name, "location_name"),
+            repository_name=check.str_param(repository_name, "repository_name"),
+            graph_name=check.str_param(graph_name, "graph_name"),
+        )
+
+    def to_graphql_input(self):
+        return {
+            "repositoryLocationName": self.location_name,
+            "repositoryName": self.repository_name,
+            "graphName": self.graph_name,
+        }
