@@ -1,19 +1,19 @@
-from collections import namedtuple
-from typing import List, NamedTuple, Optional
+from typing import Dict, List, NamedTuple, Optional
 
 from dagster import check
 from dagster.core.definitions import NodeHandle
 from dagster.core.execution.plan.inputs import (
     StepInput,
-    StepInputSourceTypes,
+    StepInputSourceUnion,
     UnresolvedCollectStepInput,
     UnresolvedMappedStepInput,
 )
 from dagster.core.execution.plan.outputs import StepOutput, StepOutputHandle, StepOutputProperties
-from dagster.core.execution.plan.plan import ExecutionPlan, StepHandleTypes
+from dagster.core.execution.plan.plan import ExecutionPlan, StepHandleTypes, StepHandleUnion
 from dagster.core.execution.plan.state import KnownExecutionState
 from dagster.core.execution.plan.step import (
     ExecutionStep,
+    IExecutionStep,
     StepKind,
     UnresolvedCollectExecutionStep,
     UnresolvedMappedExecutionStep,
@@ -27,7 +27,7 @@ from dagster.utils.error import SerializableErrorInfo
 CURRENT_SNAPSHOT_VERSION = 1
 
 
-def create_execution_plan_snapshot_id(execution_plan_snapshot):
+def create_execution_plan_snapshot_id(execution_plan_snapshot) -> str:
     check.inst_param(execution_plan_snapshot, "execution_plan_snapshot", ExecutionPlanSnapshot)
     return create_snapshot_id(execution_plan_snapshot)
 
@@ -43,25 +43,27 @@ class ExecutionPlanSnapshot(
             ("step_keys_to_execute", List[str]),
             ("initial_known_state", Optional[KnownExecutionState]),
             ("snapshot_version", Optional[int]),
-            ("step_output_versions", List[str]),
+            ("executor_name", Optional[str]),
         ],
     )
 ):
     # serdes log
+    # added executor_name
     # added step_keys_to_execute
     # added initial_known_state
     # added snapshot_version (if >=1, can be used to fully reconstruct the ExecutionPlan -
     #   can be used to track breaking changes to snapshot execution format if needed)
     # added step_output_versions
+    # removed step_output_versions
     def __new__(
         cls,
-        steps: List[str],
+        steps: List["ExecutionStepSnap"],
         artifacts_persisted: bool,
         pipeline_snapshot_id: str,
         step_keys_to_execute: Optional[List[str]] = None,
         initial_known_state: Optional[KnownExecutionState] = None,
         snapshot_version: Optional[int] = None,
-        step_output_versions: Optional[List["StepOutputVersionData"]] = None,
+        executor_name: Optional[str] = None,
     ):
         return super(ExecutionPlanSnapshot, cls).__new__(
             cls,
@@ -77,9 +79,7 @@ class ExecutionPlanSnapshot(
                 KnownExecutionState,
             ),
             snapshot_version=check.opt_int_param(snapshot_version, "snapshot_version"),
-            step_output_versions=check.opt_list_param(
-                step_output_versions, "step_output_versions", of_type=StepOutputVersionData
-            ),
+            executor_name=check.opt_str_param(executor_name, "executor_name"),
         )
 
     @property
@@ -100,37 +100,42 @@ class ExecutionPlanSnapshot(
 
 
 @whitelist_for_serdes
-class ExecutionPlanSnapshotErrorData(namedtuple("_ExecutionPlanSnapshotErrorData", "error")):
-    def __new__(cls, error):
+class ExecutionPlanSnapshotErrorData(
+    NamedTuple("_ExecutionPlanSnapshotErrorData", [("error", Optional[SerializableErrorInfo])])
+):
+    def __new__(cls, error: Optional[SerializableErrorInfo]):
         return super(ExecutionPlanSnapshotErrorData, cls).__new__(
             cls,
             error=check.opt_inst_param(error, "error", SerializableErrorInfo),
         )
 
 
-@whitelist_for_serdes
-class StepOutputVersionData(NamedTuple):
-    step_output_handle: StepOutputHandle
-    version: str
-
-
 @whitelist_for_serdes
 class ExecutionStepSnap(
-    namedtuple(
+    NamedTuple(
         "_ExecutionStepSnap",
-        "key inputs outputs solid_handle_id kind metadata_items tags step_handle",
+        [
+            ("key", str),
+            ("inputs", List["ExecutionStepInputSnap"]),
+            ("outputs", List["ExecutionStepOutputSnap"]),
+            ("solid_handle_id", str),
+            ("kind", StepKind),
+            ("metadata_items", List["ExecutionPlanMetadataItemSnap"]),
+            ("tags", Optional[Dict[str, str]]),
+            ("step_handle", Optional[StepHandleUnion]),
+        ],
     )
 ):
     def __new__(
         cls,
-        key,
-        inputs,
-        outputs,
-        solid_handle_id,
-        kind,
-        metadata_items,
-        tags=None,
-        step_handle=None,
+        key: str,
+        inputs: List["ExecutionStepInputSnap"],
+        outputs: List["ExecutionStepOutputSnap"],
+        solid_handle_id: str,
+        kind: StepKind,
+        metadata_items: List["ExecutionPlanMetadataItemSnap"],
+        tags: Optional[Dict[str, str]] = None,
+        step_handle: Optional[StepHandleUnion] = None,
     ):
         return super(ExecutionStepSnap, cls).__new__(
             cls,
@@ -142,16 +147,30 @@ class ExecutionStepSnap(
             metadata_items=check.list_param(
                 metadata_items, "metadata_items", ExecutionPlanMetadataItemSnap
             ),
-            tags=check.opt_nullable_dict_param(tags, "tags", key_type=str),
+            tags=check.opt_nullable_dict_param(tags, "tags", key_type=str, value_type=str),
             step_handle=check.opt_inst_param(step_handle, "step_handle", StepHandleTypes),
         )
 
 
 @whitelist_for_serdes
 class ExecutionStepInputSnap(
-    namedtuple("_ExecutionStepInputSnap", "name dagster_type_key upstream_output_handles source")
+    NamedTuple(
+        "_ExecutionStepInputSnap",
+        [
+            ("name", str),
+            ("dagster_type_key", str),
+            ("upstream_output_handles", List[StepOutputHandle]),
+            ("source", Optional[StepInputSourceUnion]),
+        ],
+    )
 ):
-    def __new__(cls, name, dagster_type_key, upstream_output_handles, source=None):
+    def __new__(
+        cls,
+        name: str,
+        dagster_type_key: str,
+        upstream_output_handles: List[StepOutputHandle],
+        source: Optional[StepInputSourceUnion] = None,
+    ):
         return super(ExecutionStepInputSnap, cls).__new__(
             cls,
             check.str_param(name, "name"),
@@ -159,7 +178,7 @@ class ExecutionStepInputSnap(
             check.list_param(
                 upstream_output_handles, "upstream_output_handles", of_type=StepOutputHandle
             ),
-            check.opt_inst_param(source, "source", StepInputSourceTypes),
+            check.opt_inst_param(source, "source", StepInputSourceUnion.__args__),  # type: ignore
         )
 
     @property
@@ -169,17 +188,22 @@ class ExecutionStepInputSnap(
 
 @whitelist_for_serdes
 class ExecutionStepOutputSnap(
-    namedtuple(
+    NamedTuple(
         "_ExecutionStepOutputSnap",
-        "name dagster_type_key solid_handle properties",
+        [
+            ("name", str),
+            ("dagster_type_key", str),
+            ("solid_handle", Optional[NodeHandle]),
+            ("properties", Optional[StepOutputProperties]),
+        ],
     )
 ):
     def __new__(
         cls,
-        name,
-        dagster_type_key,
-        solid_handle=None,
-        properties=None,
+        name: str,
+        dagster_type_key: str,
+        solid_handle: Optional[NodeHandle] = None,
+        properties: Optional[StepOutputProperties] = None,
     ):
         return super(ExecutionStepOutputSnap, cls).__new__(
             cls,
@@ -191,8 +215,10 @@ class ExecutionStepOutputSnap(
 
 
 @whitelist_for_serdes
-class ExecutionPlanMetadataItemSnap(namedtuple("_ExecutionPlanMetadataItemSnap", "key value")):
-    def __new__(cls, key, value):
+class ExecutionPlanMetadataItemSnap(
+    NamedTuple("_ExecutionPlanMetadataItemSnap", [("key", str), ("value", str)])
+):
+    def __new__(cls, key: str, value: str):
         return super(ExecutionPlanMetadataItemSnap, cls).__new__(
             cls,
             check.str_param(key, "key"),
@@ -226,7 +252,8 @@ def _snapshot_from_step_output(step_output):
     )
 
 
-def _snapshot_from_execution_step(execution_step):
+# def _snapshot_from_execution_step(execution_step: Union[ExecutionStep, UnresolvedMappedExecutionStep, UnresolvedCollectExecutionStep]) -> ExecutionStepSnap:
+def _snapshot_from_execution_step(execution_step: IExecutionStep) -> ExecutionStepSnap:
     check.inst_param(
         execution_step,
         "execution_step",
@@ -257,22 +284,12 @@ def _snapshot_from_execution_step(execution_step):
     )
 
 
-def snapshot_from_execution_plan(execution_plan, pipeline_snapshot_id):
+def snapshot_from_execution_plan(
+    execution_plan: ExecutionPlan, pipeline_snapshot_id: str
+) -> ExecutionPlanSnapshot:
     check.inst_param(execution_plan, "execution_plan", ExecutionPlan)
     check.str_param(pipeline_snapshot_id, "pipeline_snapshot_id")
 
-    step_output_versions = check.opt_dict_param(
-        execution_plan.step_output_versions,
-        "execution_plan.step_output_versions",
-        key_type=StepOutputHandle,
-        value_type=str,
-    )
-
-    step_output_versions_list = [
-        StepOutputVersionData(step_output_handle=step_output_handle, version=version)
-        for step_output_handle, version in step_output_versions.items()
-    ]
-
     return ExecutionPlanSnapshot(
         steps=sorted(
             list(map(_snapshot_from_execution_step, execution_plan.steps)), key=lambda es: es.key
@@ -282,5 +299,5 @@ def snapshot_from_execution_plan(execution_plan, pipeline_snapshot_id):
         step_keys_to_execute=execution_plan.step_keys_to_execute,
         initial_known_state=execution_plan.known_state,
         snapshot_version=CURRENT_SNAPSHOT_VERSION,
-        step_output_versions=step_output_versions_list,
+        executor_name=execution_plan.executor_name,
     )
