@@ -2,7 +2,7 @@ from contextlib import contextmanager
 
 from sqlalchemy.pool import NullPool
 
-from dagster import check
+from dagster import StringSource, check
 from dagster.core.storage.sql import (
     check_alembic_revision,
     create_engine,
@@ -11,30 +11,33 @@ from dagster.core.storage.sql import (
     run_alembic_upgrade,
     stamp_alembic_rev,
 )
-from dagster.core.storage.sqlite import create_db_conn_string
+from dagster.core.storage.sqlite import create_db_conn_string, get_sqlite_version
 from dagster.serdes import ConfigurableClass, ConfigurableClassData
 from dagster.utils import mkdir_p
 
 from ..schema import ScheduleStorageSqlMetadata
 from ..sql_schedule_storage import SqlScheduleStorage
 
+MINIMUM_SQLITE_BATCH_VERSION = "3.25.0"
+
 
 class SqliteScheduleStorage(SqlScheduleStorage, ConfigurableClass):
-    """Local SQLite backed schedule storage
-    """
+    """Local SQLite backed schedule storage"""
 
     def __init__(self, conn_string, inst_data=None):
         check.str_param(conn_string, "conn_string")
         self._conn_string = conn_string
         self._inst_data = check.opt_inst_param(inst_data, "inst_data", ConfigurableClassData)
 
+        super().__init__()
+
     @property
     def inst_data(self):
         return self._inst_data
 
     @classmethod
     def config_type(cls):
-        return {"base_dir": str}
+        return {"base_dir": StringSource}
 
     @staticmethod
     def from_config_value(inst_data, config_value):
@@ -46,13 +49,14 @@ class SqliteScheduleStorage(SqlScheduleStorage, ConfigurableClass):
         mkdir_p(base_dir)
         conn_string = create_db_conn_string(base_dir, "schedules")
         engine = create_engine(conn_string, poolclass=NullPool)
-        engine.execute("PRAGMA journal_mode=WAL;")
-        ScheduleStorageSqlMetadata.create_all(engine)
-        connection = engine.connect()
         alembic_config = get_alembic_config(__file__)
-        db_revision, head_revision = check_alembic_revision(alembic_config, connection)
-        if not (db_revision and head_revision):
-            stamp_alembic_rev(alembic_config, engine)
+
+        with engine.connect() as connection:
+            db_revision, head_revision = check_alembic_revision(alembic_config, connection)
+            if not (db_revision and head_revision):
+                ScheduleStorageSqlMetadata.create_all(engine)
+                engine.execute("PRAGMA journal_mode=WAL;")
+                stamp_alembic_rev(alembic_config, connection)
 
         return SqliteScheduleStorage(conn_string, inst_data)
 
@@ -64,12 +68,15 @@ class SqliteScheduleStorage(SqlScheduleStorage, ConfigurableClass):
             with handle_schema_errors(
                 conn,
                 get_alembic_config(__file__),
-                msg="Sqlite schedule storage requires migration",
             ):
                 yield conn
         finally:
             conn.close()
 
+    @property
+    def supports_batch_queries(self):
+        return get_sqlite_version() > MINIMUM_SQLITE_BATCH_VERSION
+
     def upgrade(self):
         alembic_config = get_alembic_config(__file__)
         with self.connect() as conn:
