@@ -2,21 +2,25 @@ import datetime
 import os
 import sys
 import time
+from typing import cast
 
 import pendulum
+
 from dagster import check
+from dagster.core.definitions.schedule_definition import DefaultScheduleStatus
 from dagster.core.errors import DagsterUserCodeUnreachableError
-from dagster.core.host_representation import PipelineSelector, RepositoryLocation
+from dagster.core.host_representation import ExternalSchedule, PipelineSelector
 from dagster.core.instance import DagsterInstance
 from dagster.core.scheduler.instigation import (
     InstigatorState,
     InstigatorStatus,
     InstigatorType,
+    ScheduleInstigatorData,
     TickData,
     TickStatus,
 )
 from dagster.core.scheduler.scheduler import DEFAULT_MAX_CATCHUP_RUNS, DagsterSchedulerError
-from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus, PipelineRunsFilter
+from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus, RunsFilter
 from dagster.core.storage.tags import RUN_KEY_TAG, SCHEDULED_EXECUTION_TIME_TAG, check_tags
 from dagster.core.telemetry import SCHEDULED_RUN_CREATED, hash_name, log_action
 from dagster.core.workspace import IWorkspace
@@ -34,7 +38,7 @@ class _ScheduleLaunchContext:
 
     @property
     def failure_count(self) -> int:
-        return self._tick.job_tick_data.failure_count
+        return self._tick.tick_data.failure_count
 
     def update_state(self, status, error=None, **kwargs):
         skip_reason = kwargs.get("skip_reason")
@@ -46,11 +50,11 @@ class _ScheduleLaunchContext:
         if skip_reason:
             self._tick = self._tick.with_reason(skip_reason=skip_reason)
 
-    def add_run(self, run_id, run_key=None):
-        self._tick = self._tick.with_run(run_id, run_key)
+    def add_run_info(self, run_id=None, run_key=None):
+        self._tick = self._tick.with_run_info(run_id, run_key)
 
     def _write(self):
-        self._instance.update_job_tick(self._tick)
+        self._instance.update_tick(self._tick)
 
     def __enter__(self):
         return self
@@ -107,45 +111,132 @@ def launch_scheduled_runs(
     check.inst_param(instance, "instance", DagsterInstance)
     check.inst_param(workspace, "workspace", IWorkspace)
 
-    schedules = [
-        s
-        for s in instance.all_stored_job_state(job_type=InstigatorType.SCHEDULE)
-        if s.status == InstigatorStatus.RUNNING
-    ]
+    workspace_snapshot = {
+        location_entry.origin: location_entry
+        for location_entry in workspace.get_workspace_snapshot().values()
+    }
+
+    all_schedule_states = {
+        schedule_state.origin.get_id(): schedule_state
+        for schedule_state in instance.all_instigator_state(instigator_type=InstigatorType.SCHEDULE)
+    }
+
+    schedules = {}
+    for location_entry in workspace_snapshot.values():
+        repo_location = location_entry.repository_location
+        if repo_location:
+            for repo in repo_location.get_repositories().values():
+                for schedule in repo.get_external_schedules():
+                    origin_id = schedule.get_external_origin().get_id()
+                    if schedule.get_current_instigator_state(
+                        all_schedule_states.get(origin_id)
+                    ).is_running:
+                        schedules[origin_id] = schedule
+        elif location_entry.load_error and log_verbose_checks:
+            logger.warning(
+                f"Could not load location {location_entry.origin.location_name} to check for schedules due to the following error: {location_entry.load_error}"
+            )
+
+    # Remove any schedule states that were previously created with AUTOMATICALLY_RUNNING
+    # and can no longer be found in the workspace (so that if they are later added
+    # back again, their timestamps will start at the correct place)
+    schedule_state_ids_to_delete = {
+        origin_id
+        for origin_id, schedule_state in all_schedule_states.items()
+        if origin_id not in schedules
+        and schedule_state.status == InstigatorStatus.AUTOMATICALLY_RUNNING
+    }
+    for origin_id in schedule_state_ids_to_delete:
+        instance.schedule_storage.delete_instigator_state(origin_id)
+
+    if log_verbose_checks:
+        unloadable_schedule_states = {
+            origin_id: schedule_state
+            for origin_id, schedule_state in all_schedule_states.items()
+            if origin_id not in schedules and schedule_state.status == InstigatorStatus.RUNNING
+        }
+
+        for schedule_state in unloadable_schedule_states.values():
+            schedule_name = schedule_state.origin.instigator_name
+            repo_location_origin = (
+                schedule_state.origin.external_repository_origin.repository_location_origin
+            )
+
+            repo_location_name = repo_location_origin.location_name
+            repo_name = schedule_state.origin.external_repository_origin.repository_name
+            if (
+                repo_location_origin not in workspace_snapshot
+                or not workspace_snapshot[repo_location_origin].repository_location
+            ):
+                logger.warning(
+                    f"Schedule {schedule_name} was started from a location "
+                    f"{repo_location_name} that can no longer be found in the workspace, or has "
+                    "metadata that has changed since the schedule was started. You can turn off "
+                    "this schedule in the Dagit UI from the Status tab."
+                )
+            elif not workspace_snapshot[repo_location_origin].repository_location.has_repository(
+                repo_name
+            ):
+                logger.warning(
+                    f"Could not find repository {repo_name} in location {repo_location_name} to "
+                    + f"run schedule {schedule_name}. If this repository no longer exists, you can "
+                    + "turn off the schedule in the Dagit UI from the Status tab.",
+                )
+            else:
+                logger.warning(
+                    f"Could not find schedule {schedule_name} in repository {repo_name}. If this "
+                    "schedule no longer exists, you can turn it off in the Dagit UI from the "
+                    "Status tab.",
+                )
 
     if not schedules:
-        if log_verbose_checks:
-            # Only log the "No schedules have been started" warning once per workspace reload
-            # to avoid spamming the logs
-            logger.info("Not checking for any runs since no schedules have been started.")
+        logger.debug("Not checking for any runs since no schedules have been started.")
         yield
         return
 
     if log_verbose_checks:
-        schedule_names = ", ".join([schedule.job_name for schedule in schedules])
+        schedule_names = ", ".join([schedule.name for schedule in schedules.values()])
         logger.info(f"Checking for new runs for the following schedules: {schedule_names}")
 
-    for schedule_state in schedules:
+    for external_schedule in schedules.values():
         error_info = None
         try:
-            origin = schedule_state.origin.external_repository_origin.repository_location_origin
-            repo_location = workspace.get_location(origin)
+            schedule_state = all_schedule_states.get(
+                external_schedule.get_external_origin().get_id()
+            )
+            if not schedule_state:
+                assert external_schedule.default_status == DefaultScheduleStatus.RUNNING
+                schedule_state = InstigatorState(
+                    external_schedule.get_external_origin(),
+                    InstigatorType.SCHEDULE,
+                    InstigatorStatus.AUTOMATICALLY_RUNNING,
+                    ScheduleInstigatorData(
+                        external_schedule.cron_schedule,
+                        end_datetime_utc.timestamp(),
+                    ),
+                )
+                instance.add_instigator_state(schedule_state)
+
             yield from launch_scheduled_runs_for_schedule(
                 instance,
                 logger,
+                external_schedule,
                 schedule_state,
                 workspace,
-                repo_location,
                 end_datetime_utc,
                 max_catchup_runs,
                 max_tick_retries,
-                (debug_crash_flags.get(schedule_state.job_name) if debug_crash_flags else None),
+                (
+                    debug_crash_flags.get(schedule_state.instigator_name)
+                    if debug_crash_flags
+                    else None
+                ),
                 log_verbose_checks=log_verbose_checks,
             )
         except Exception:
             error_info = serializable_error_info_from_exc_info(sys.exc_info())
             logger.error(
-                f"Scheduler caught an error for schedule {schedule_state.job_name} : {error_info.to_string()}"
+                f"Scheduler caught an error for schedule {external_schedule.name} : {error_info.to_string()}"
             )
         yield error_info
 
@@ -153,56 +244,47 @@ def launch_scheduled_runs(
 def launch_scheduled_runs_for_schedule(
     instance,
     logger,
-    schedule_state,
+    external_schedule: ExternalSchedule,
+    schedule_state: InstigatorState,
     workspace,
-    repo_location,
-    end_datetime_utc,
+    end_datetime_utc: datetime.datetime,
     max_catchup_runs,
     max_tick_retries,
     debug_crash_flags=None,
     log_verbose_checks=True,
 ):
-    check.inst_param(instance, "instance", DagsterInstance)
-    check.inst_param(schedule_state, "schedule_state", InstigatorState)
-    check.inst_param(end_datetime_utc, "end_datetime_utc", datetime.datetime)
-    check.inst_param(repo_location, "repo_location", RepositoryLocation)
+    instance = check.inst_param(instance, "instance", DagsterInstance)
+    schedule_state = check.opt_inst_param(schedule_state, "schedule_state", InstigatorState)
+    end_datetime_utc = check.inst_param(end_datetime_utc, "end_datetime_utc", datetime.datetime)
 
-    latest_tick = instance.get_latest_job_tick(schedule_state.job_origin_id)
+    instigator_origin_id = external_schedule.get_external_origin_id()
+    ticks = instance.get_ticks(instigator_origin_id, limit=1)
+    latest_tick = ticks[0] if ticks else None
 
-    start_timestamp_utc = schedule_state.job_specific_data.start_timestamp
+    instigator_data = cast(ScheduleInstigatorData, schedule_state.instigator_data)
+    start_timestamp_utc = instigator_data.start_timestamp if schedule_state else None
 
     if latest_tick:
-        if latest_tick.status == TickStatus.STARTED:
-            # Scheduler was interrupted while performing this tick, re-do it
-            start_timestamp_utc = max(start_timestamp_utc, latest_tick.timestamp)
-        elif (
+        if latest_tick.status == TickStatus.STARTED or (
             latest_tick.status == TickStatus.FAILURE
             and latest_tick.failure_count <= max_tick_retries
         ):
-            # Tick failed and hasn't reached its retry limit, retry it
-            start_timestamp_utc = max(start_timestamp_utc, latest_tick.timestamp)
+            # Scheduler was interrupted while performing this tick, re-do it
+            start_timestamp_utc = (
+                max(start_timestamp_utc, latest_tick.timestamp)
+                if start_timestamp_utc
+                else latest_tick.timestamp
+            )
         else:
-            start_timestamp_utc = max(start_timestamp_utc, latest_tick.timestamp + 1)
+            start_timestamp_utc = (
+                max(start_timestamp_utc, latest_tick.timestamp + 1)
+                if start_timestamp_utc
+                else latest_tick.timestamp + 1
+            )
+    else:
+        start_timestamp_utc = instigator_data.start_timestamp
 
-    schedule_name = schedule_state.job_name
-    repo_name = schedule_state.origin.external_repository_origin.repository_name
-
-    if not repo_location.has_repository(repo_name):
-        raise DagsterSchedulerError(
-            f"Could not find repository {repo_name} in location {repo_location.name} to "
-            + f"run schedule {schedule_name}. If this repository no longer exists, you can "
-            + "turn off the schedule in the Dagit UI.",
-        )
-
-    external_repo = repo_location.get_repository(repo_name)
-
-    if not external_repo.has_external_schedule(schedule_name):
-        raise DagsterSchedulerError(
-            f"Could not find schedule {schedule_name} in repository {repo_name}. If this "
-            "schedule no longer exists, you can turn it off in the Dagit UI.",
-        )
-
-    external_schedule = external_repo.get_external_schedule(schedule_name)
+    schedule_name = external_schedule.name
 
     timezone_str = external_schedule.execution_timezone
     if not timezone_str:
@@ -251,11 +333,11 @@ def launch_scheduled_runs_for_schedule(
                     f"Resuming previously interrupted schedule execution at {schedule_time_str}"
                 )
         else:
-            tick = instance.create_job_tick(
+            tick = instance.create_tick(
                 TickData(
-                    job_origin_id=external_schedule.get_external_origin_id(),
-                    job_name=schedule_name,
-                    job_type=InstigatorType.SCHEDULE,
+                    instigator_origin_id=instigator_origin_id,
+                    instigator_name=schedule_name,
+                    instigator_type=InstigatorType.SCHEDULE,
                     status=TickStatus.STARTED,
                     timestamp=schedule_timestamp,
                 )
@@ -271,8 +353,6 @@ def launch_scheduled_runs_for_schedule(
                     instance,
                     logger,
                     workspace,
-                    repo_location,
-                    external_repo,
                     external_schedule,
                     schedule_time,
                     tick_context,
@@ -323,8 +403,6 @@ def _schedule_runs_at_time(
     instance,
     logger,
     workspace,
-    repo_location,
-    external_repo,
     external_schedule,
     schedule_time,
     tick_context,
@@ -332,18 +410,25 @@ def _schedule_runs_at_time(
 ):
     schedule_name = external_schedule.name
 
+    schedule_origin = external_schedule.get_external_origin()
+    repository_handle = external_schedule.handle.repository_handle
+
     pipeline_selector = PipelineSelector(
-        location_name=repo_location.name,
-        repository_name=external_repo.name,
+        location_name=schedule_origin.external_repository_origin.repository_location_origin.location_name,
+        repository_name=schedule_origin.external_repository_origin.repository_name,
         pipeline_name=external_schedule.pipeline_name,
         solid_selection=external_schedule.solid_selection,
     )
 
+    repo_location = workspace.get_location(
+        schedule_origin.external_repository_origin.repository_location_origin.location_name
+    )
+
     external_pipeline = repo_location.get_external_pipeline(pipeline_selector)
 
     schedule_execution_data = repo_location.get_external_schedule_execution_data(
         instance=instance,
-        repository_handle=external_repo.handle,
+        repository_handle=repository_handle,
         schedule_name=external_schedule.name,
         scheduled_execution_time=schedule_time,
     )
@@ -374,7 +459,7 @@ def _schedule_runs_at_time(
                 logger.info(
                     f"Run {run.run_id} already completed for this execution of {external_schedule.name}"
                 )
-                tick_context.add_run(run_id=run.run_id, run_key=run_request.run_key)
+                tick_context.add_run_info(run_id=run.run_id, run_key=run_request.run_key)
                 yield
                 continue
             else:
@@ -405,7 +490,7 @@ def _schedule_runs_at_time(
                 yield error_info
 
         _check_for_debug_crash(debug_crash_flags, "RUN_LAUNCHED")
-        tick_context.add_run(run_id=run.run_id, run_key=run_request.run_key)
+        tick_context.add_run_info(run_id=run.run_id, run_key=run_request.run_key)
         _check_for_debug_crash(debug_crash_flags, "RUN_ADDED")
         yield
 
@@ -422,7 +507,7 @@ def _get_existing_run_for_request(instance, external_schedule, schedule_time, ru
     )
     if run_request.run_key:
         tags[RUN_KEY_TAG] = run_request.run_key
-    runs_filter = PipelineRunsFilter(tags=tags)
+    runs_filter = RunsFilter(tags=tags)
     existing_runs = instance.get_runs(runs_filter)
     if not len(existing_runs):
         return None
@@ -462,11 +547,11 @@ def _create_scheduler_run(
     log_action(
         instance,
         SCHEDULED_RUN_CREATED,
-        repo_hash=hash_name(repo_location.name),
-        pipeline_name_hash=hash_name(external_pipeline.name),
         metadata={
             "DAEMON_SESSION_ID": get_telemetry_daemon_session_id(),
             "SCHEDULE_NAME_HASH": hash_name(external_schedule.name),
+            "repo_hash": hash_name(repo_location.name),
+            "pipeline_name_hash": hash_name(external_pipeline.name),
         },
     )
 
