@@ -1,8 +1,6 @@
 """Internal py2/3 compatibility library. A little more than six."""
-
 import datetime
 import inspect
-import multiprocessing
 import os
 import shlex
 import signal
@@ -11,11 +9,12 @@ import tempfile
 import threading
 import time
 from contextlib import contextmanager
-from types import MethodType
+from datetime import timezone
+from types import MethodType, ModuleType
 
 import pendulum
-import pkg_resources
 
+from .compat.pendulum import PendulumDateTime
 from .json import JSONDecodeError, dump, dumps
 from .temp_dir import get_system_temp_directory
 
@@ -23,14 +22,12 @@ IS_WINDOWS = os.name == "nt"
 
 funcsigs = inspect
 
-multiprocessing = multiprocessing.get_context("spawn")  # type: ignore[assignment]
-
 IS_WINDOWS = os.name == "nt"
 
 # TODO implement a generic import by name -- see https://stackoverflow.com/questions/301134/how-to-import-a-module-given-its-name
 
 # https://stackoverflow.com/a/67692/324449
-def import_module_from_path(module_name, path_to_file):
+def import_module_from_path(module_name: str, path_to_file: str) -> ModuleType:
     import importlib.util
 
     spec = importlib.util.spec_from_file_location(module_name, path_to_file)
@@ -100,6 +97,8 @@ def wait_for_process(process, timeout=30):
 
 
 def kill_process(process):
+    import multiprocessing
+
     if not isinstance(process, multiprocessing.Process):
         raise Exception("invalid process argument passed to kill_process")
 
@@ -124,12 +123,6 @@ def builtin_print():
     return "builtins.print"
 
 
-def get_utc_timezone():
-    from datetime import timezone
-
-    return timezone.utc
-
-
 def get_current_datetime_in_utc():
     return pendulum.now("UTC")
 
@@ -139,7 +132,7 @@ def get_timestamp_from_utc_datetime(utc_datetime):
     if isinstance(utc_datetime, PendulumDateTime):
         return utc_datetime.timestamp()
 
-    if utc_datetime.tzinfo != get_utc_timezone():
+    if utc_datetime.tzinfo != timezone.utc:
         raise Exception("Must pass in a UTC timezone to compute UNIX timestamp")
 
     return utc_datetime.timestamp()
@@ -172,47 +165,3 @@ def get_import_error_message(import_error):
 @contextmanager
 def nullcontext():
     yield
-
-
-_IS_PENDULUM_2 = pkg_resources.get_distribution("pendulum").version.split(".")[0] == "2"
-
-
-@contextmanager
-def mock_pendulum_timezone(override_timezone):
-    if _IS_PENDULUM_2:
-        with pendulum.tz.test_local_timezone(  # pylint: disable=no-member
-            pendulum.tz.timezone(override_timezone)  # pylint: disable=no-member
-        ):
-
-            yield
-    else:
-        with pendulum.tz.LocalTimezone.test(  # pylint: disable=no-member
-            pendulum.Timezone.load(override_timezone)  # pylint: disable=no-member
-        ):
-
-            yield
-
-
-def create_pendulum_time(year, month, day, *args, **kwargs):
-    return (
-        pendulum.datetime(  # pylint: disable=no-member, pendulum-create
-            year, month, day, *args, **kwargs
-        )
-        if _IS_PENDULUM_2
-        else pendulum.create(  # pylint: disable=no-member, pendulum-create
-            year, month, day, *args, **kwargs
-        )
-    )
-
-
-PendulumDateTime = (
-    pendulum.DateTime if _IS_PENDULUM_2 else pendulum.Pendulum  # pylint: disable=no-member
-)
-
-# Workaround for issues with .in_tz() in pendulum:
-# https://github.com/sdispater/pendulum/issues/535
-def to_timezone(dt, tz):
-    from dagster import check
-
-    check.inst_param(dt, "dt", PendulumDateTime)
-    return pendulum.from_timestamp(dt.timestamp(), tz=tz)
